@@ -253,15 +253,10 @@ fi
         -DWITH_PAM=ON \
         -DWITH_ROCKSDB=ON \
         -DWITH_INNODB_MEMCACHED=ON \
+        -DWITH_ZLIB=system \
+        -DWITH_NUMA=ON \
         -DDOWNLOAD_BOOST=1 \
-	-DFORCE_INSOURCE_BUILD=1 \
-	-DWITH_SYSTEM_LIBS=ON \
-        -DWITH_PROTOBUF=bundled \
-        -DWITH_RAPIDJSON=bundled \
-        -DWITH_ICU=bundled \
-        -DWITH_LZ4=bundled \
-        -DWITH_EDITLINE=bundled \
-	-DWITH_NUMA=ON \
+	      -DFORCE_INSOURCE_BUILD=1 \
         -DWITH_BOOST="$WORKDIR_ABS/libboost" \
         $WITH_MECAB_OPTION $OPENSSL_INCLUDE $OPENSSL_LIBRARY $CRYPTO_LIBRARY
 
