@@ -77,6 +77,7 @@ static char *audit_log_exclude_databases = nullptr;
 static char *audit_log_include_databases = nullptr;
 static char *audit_log_exclude_commands = nullptr;
 static char *audit_log_include_commands = nullptr;
+std::atomic<uint64_t> audit_log_buffer_size_overflow(0);
 
 PSI_memory_key key_memory_audit_log_logger_handle;
 PSI_memory_key key_memory_audit_log_handler;
@@ -1711,8 +1712,19 @@ static st_mysql_audit audit_log_descriptor = {
 /*
   Plugin status variables for SHOW STATUS
 */
-
+static int show_audit_log_buffer_size_overflow(THD *, SHOW_VAR *var,
+                                               char *buff) {
+  var->type = SHOW_LONG;
+  var->value = buff;
+  uint64_t *value = reinterpret_cast<uint64_t *>(buff);
+  *value = static_cast<uint64_t>(
+      audit_log_buffer_size_overflow.load(std::memory_order_relaxed));
+  return 0;
+}
 static SHOW_VAR audit_log_status_variables[] = {
+    {"Audit_log_buffer_size_overflow",
+     (char *)&show_audit_log_buffer_size_overflow, SHOW_FUNC,
+     SHOW_SCOPE_GLOBAL},
     {NullS, NullS, SHOW_LONG, SHOW_SCOPE_GLOBAL}};
 
 /*
