@@ -14,8 +14,8 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
 
+#include <atomic>
 #include "i_keyring_key.h"
-
 #include "my_dbug.h"
 
 namespace keyring {
@@ -55,16 +55,16 @@ class System_key_adapter : public IKey {
   virtual uchar *get_key_data() {
     DBUG_ASSERT(keyring_key != nullptr);
 
-    if (system_key_data.get_key_data() == nullptr) construct_system_key_data();
+    if (system_key_data.key_data.load() == nullptr) construct_system_key_data();
 
-    return system_key_data.get_key_data();
+    return system_key_data.key_data.load();
   }
   virtual size_t get_key_data_size() {
     DBUG_ASSERT(keyring_key != nullptr);
 
-    if (system_key_data.get_key_data() == nullptr) construct_system_key_data();
+    if (system_key_data.key_data.load() == nullptr) construct_system_key_data();
 
-    return system_key_data.get_key_data_size();
+    return system_key_data.key_data_size;
   }
   virtual size_t get_key_pod_size() const noexcept {
     DBUG_ASSERT(false);
@@ -74,6 +74,7 @@ class System_key_adapter : public IKey {
     DBUG_ASSERT(false);
     return nullptr;
   }
+  virtual void xor_data(uchar *, size_t) noexcept { DBUG_ASSERT(false); }
   virtual void xor_data() noexcept { DBUG_ASSERT(false); }
   virtual void set_key_data(uchar *key_data, size_t key_data_size) {
     keyring_key->set_key_data(key_data, key_data_size);
@@ -116,17 +117,13 @@ class System_key_adapter : public IKey {
     System_key_data();
     ~System_key_data();
 
-    bool allocate(size_t key_data_size);
     void free();
-    uchar *get_key_data();
-    size_t get_key_data_size();
 
-   private:
-    uchar *key_data;
+    std::atomic<uchar *> key_data;
     size_t key_data_size;
   };
 
-  void construct_system_key_data();
+  void construct_system_key_data() noexcept;
 
   System_key_data system_key_data;
   uint key_version;
