@@ -32,7 +32,7 @@ Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved.
     along with Percona TokuBackup.  If not, see <http://www.gnu.org/licenses/>.
 ======= */
 
-#ident "$Id: 1114ba3ad232b3fa2d372033eafdbc661d501840 $"
+#ident "$Id: 04e43a1fb2835644491f5c3542d7402e1c1b1763 $"
 
 #include <stdio.h> // rename(),
 #include <fcntl.h> // open()
@@ -51,7 +51,7 @@ Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved.
 #include "backup_debug.h"
 #include "directory_set.h"
 
-#if DEBUG_HOTBACKUP
+#if defined(DEBUG_HOTBACKUP)
 #define WARN(string, arg) HotBackup::InterposeWarn(string, arg);
 #define TRACE(string, arg) HotBackup::InterposeTrace(string, arg);
 #define ERROR(string, arg) HotBackup::InterposeError(string, arg);
