@@ -32,7 +32,7 @@ Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved.
     along with Percona TokuBackup.  If not, see <http://www.gnu.org/licenses/>.
 ======= */
 
-#ident "$Id: 51d5d83578dc935c5c1e0e36b6925cc7f66eaea3 $"
+#ident "$Id: 07129478e78250144f9f29b42a6fa381fe8553fc $"
 
 #include "backup_debug.h"
 #include "file_hash_table.h"
@@ -56,7 +56,7 @@ Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved.
 #include <unistd.h>
 #include "backup_helgrind.h"
 
-#if defined(DEBUG_HOTBACKUP) && DEBUG_HOTBACKUP
+#if defined(DEBUG_HOTBACKUP)
 #define WARN(string, arg) HotBackup::CaptureWarn(string, arg)
 #define TRACE(string, arg) HotBackup::CaptureTrace(string, arg)
 #define ERROR(string, arg) HotBackup::CaptureError(string, arg)
@@ -367,9 +367,8 @@ int manager::open(int fd, const char *file, int flags) throw() {
     // First, get the description and source_file objects associated
     // with the given fd.
     m_map.get(fd, &description, BACKTRACE(NULL));
+    
     source = description->get_source_file();
-
-    with_file_hash_table_mutex mtl(&m_table);
     with_source_file_name_read_lock sfl(source);
 
     // Next, determine the full path of the backup file.
@@ -638,7 +637,7 @@ int manager::rename(const char *oldpath, const char *newpath) throw() {
         }
     }
 
-    free(const_cast<char*>(full_old_path));
+    free((void*)full_old_path);
     TRACE("rename() exiting...", oldpath);
     return user_error;
 }
