@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2013, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2013, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -22,42 +22,42 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
-#include "connection_handler_impl.h"
+#include <stddef.h>
 
-#include "channel_info.h"                // Channel_info
-#include "connection_handler_manager.h"  // Connection_handler_manager
-#include "mysqld_error.h"                // ER_*
-#include "mysqld_thd_manager.h"          // Global_THD_manager
-#include "sql_audit.h"                   // mysql_audit_release
-#include "sql_connect.h"                 // close_connection
-#include "sql_class.h"                   // THD
-#include "sql_parse.h"                   // do_command
-#include "sql_thd_internal_api.h"        // thd_set_thread_stack
+#include "my_systime.h"  //my_getsystime
+#include "mysql/psi/mysql_socket.h"
+#include "mysql/psi/mysql_thread.h"
+#include "mysql_com.h"
+#include "mysqld_error.h"                   // ER_*
+#include "sql/conn_handler/channel_info.h"  // Channel_info
+#include "sql/conn_handler/connection_handler_impl.h"
+#include "sql/conn_handler/connection_handler_manager.h"  // Connection_handler_manager
+#include "sql/mysqld.h"              // connection_errors_internal
+#include "sql/mysqld_thd_manager.h"  // Global_THD_manager
+#include "sql/protocol_classic.h"
+#include "sql/sql_class.h"             // THD
+#include "sql/sql_connect.h"           // close_connection
+#include "sql/sql_parse.h"             // do_command
+#include "sql/sql_thd_internal_api.h"  // thd_set_thread_stack
 
-
-bool One_thread_connection_handler::add_connection(Channel_info* channel_info)
-{
-  if (my_thread_init())
-  {
+bool One_thread_connection_handler::add_connection(Channel_info *channel_info) {
+  if (my_thread_init()) {
     connection_errors_internal++;
     channel_info->send_error_and_close_channel(ER_OUT_OF_RESOURCES, 0, false);
-    Connection_handler_manager::dec_connection_count(false);
+    Connection_handler_manager::dec_connection_count();
     return true;
   }
 
-  THD* thd= channel_info->create_thd();
-  if (thd == NULL)
-  {
+  THD *thd = channel_info->create_thd();
+  if (thd == NULL) {
     connection_errors_internal++;
     channel_info->send_error_and_close_channel(ER_OUT_OF_RESOURCES, 0, false);
-    Connection_handler_manager::dec_connection_count(false);
+    Connection_handler_manager::dec_connection_count();
     return true;
   }
 
   thd->set_new_thread_id();
 
-  thd->start_utime= thd->thr_create_utime= my_micro_time();
-
   /*
     handle_one_connection() is normally the only way a thread would
     start and would always be on the very high end of the stack ,
@@ -66,51 +66,38 @@ bool One_thread_connection_handler::add_connection(Channel_info* channel_info)
     need to know the start of the stack so that we could check for
     stack overruns.
   */
-  thd_set_thread_stack(thd, (char*) &thd);
-  if (thd->store_globals())
-  {
-    close_connection(thd, ER_OUT_OF_RESOURCES);
-    thd->release_resources();
-    delete thd;
-    Connection_handler_manager::dec_connection_count(false);
-    return true;
-  }
+  thd_set_thread_stack(thd, (char *)&thd);
+  thd->store_globals();
 
   mysql_thread_set_psi_id(thd->thread_id());
   mysql_socket_set_thread_owner(
-    thd->get_protocol_classic()->get_vio()->mysql_socket);
+      thd->get_protocol_classic()->get_vio()->mysql_socket);
 
-  Global_THD_manager *thd_manager= Global_THD_manager::get_instance();
+  Global_THD_manager *thd_manager = Global_THD_manager::get_instance();
   thd_manager->add_thd(thd);
 
-  bool error= false;
-  bool create_user= true;
-  if (thd_prepare_connection(thd, false))
-  {
-    error= true; // Returning true causes inc_aborted_connects() to be called.
-    create_user= false;
-  }
-  else
-  {
+  bool error = false;
+  bool create_user = true;
+  if (thd_prepare_connection(thd)) {
+    error = true;  // Returning true causes inc_aborted_connects() to be called.
+    create_user = false;
+  } else {
     delete channel_info;
-    while (thd_connection_alive(thd))
-    {
-      if (do_command(thd))
-        break;
+    while (thd_connection_alive(thd)) {
+      if (do_command(thd)) break;
     }
     end_connection(thd);
   }
   close_connection(thd, 0, false, false);
 
-  if (unlikely(opt_userstat))
-  {
+  if (unlikely(opt_userstat)) {
     thd->update_stats(false);
-    update_global_user_stats(thd, create_user, time(NULL));
+    update_global_user_stats(thd, create_user, my_getsystime());
   }
 
   thd->release_resources();
   thd_manager->remove_thd(thd);
-  Connection_handler_manager::dec_connection_count(false);
+  Connection_handler_manager::dec_connection_count();
   delete thd;
   return error;
 }
