@@ -1,4 +1,4 @@
-/* Copyright (c) 2006, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2006, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -20,48 +20,75 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
-#include "event_scheduler.h"
+#include "sql/event_scheduler.h"
 
-#include "events.h"
-#include "event_data_objects.h"
-#include "event_queue.h"
-#include "event_db_repository.h"
-#include "auth_common.h"             // SUPER_ACL
-#include "log.h"
-#include "mysqld_thd_manager.h"      // Global_THD_manager
-#include "sql_error.h"               // Sql_condition
-#include "sql_class.h"               // THD
+#include <stdio.h>
+#include <string.h>
+
+#include "lex_string.h"
+#include "m_ctype.h"
+#include "m_string.h"
+#include "my_command.h"
+#include "my_dbug.h"
+#include "my_loglevel.h"
+#include "my_psi_config.h"
+#include "my_sys.h"
+#include "my_thread.h"
+#include "mysql/components/services/log_builtins.h"
+#include "mysql/components/services/psi_statement_bits.h"
+#include "mysql/psi/mysql_cond.h"
+#include "mysql/psi/mysql_mutex.h"
+#include "mysql/psi/mysql_statement.h"
+#include "mysql/psi/mysql_thread.h"
+#include "mysql/service_mysql_alloc.h"
+#include "mysql/thread_type.h"
+#include "mysql_com.h"
+#include "mysqld_error.h"
+#include "sql/auth/auth_acls.h"
+#include "sql/auth/sql_security_ctx.h"
+#include "sql/current_thd.h"
+#include "sql/dd/dd_schema.h"  // dd::Schema_MDL_locker
+#include "sql/dd/types/event.h"
+#include "sql/event_data_objects.h"
+#include "sql/event_db_repository.h"
+#include "sql/event_queue.h"
+#include "sql/events.h"
+#include "sql/log.h"
+#include "sql/mdl.h"
+#include "sql/mysqld.h"              // my_localhost slave_net_timeout
+#include "sql/mysqld_thd_manager.h"  // Global_THD_manager
+#include "sql/protocol_classic.h"
+#include "sql/psi_memory_key.h"
+#include "sql/query_options.h"
+#include "sql/sql_class.h"  // THD
+#include "sql/sql_const.h"
+#include "sql/sql_error.h"  // Sql_condition
+#include "sql/system_variables.h"
+#include "sql_string.h"
+#include "thr_mutex.h"
 
 /**
   @addtogroup Event_Scheduler
   @{
 */
 
-#define LOCK_DATA()       lock_data(__func__, __LINE__)
-#define UNLOCK_DATA()     unlock_data(__func__, __LINE__)
+#define LOCK_DATA() lock_data(__func__, __LINE__)
+#define UNLOCK_DATA() unlock_data(__func__, __LINE__)
 #define COND_STATE_WAIT(mythd, abstime, stage) \
-        cond_wait(mythd, abstime, stage, __func__, __FILE__, __LINE__)
+  cond_wait(mythd, abstime, stage, __func__, __FILE__, __LINE__)
 
 extern my_thread_attr_t connection_attrib;
 
-
-Event_db_repository *Event_worker_thread::db_repository;
-
-
-static
-const LEX_STRING scheduler_states_names[] =
-{
-  { C_STRING_WITH_LEN("INITIALIZED") },
-  { C_STRING_WITH_LEN("RUNNING") },
-  { C_STRING_WITH_LEN("STOPPING") }
-};
+static const LEX_STRING scheduler_states_names[] = {
+    {C_STRING_WITH_LEN("INITIALIZED")},
+    {C_STRING_WITH_LEN("RUNNING")},
+    {C_STRING_WITH_LEN("STOPPING")}};
 
 struct scheduler_param {
   THD *thd;
   Event_scheduler *scheduler;
 };
 
-
 /*
   Prints the stack of infos, warnings, errors from thd to
   the console so it can be fetched by the logs-into-tables and
@@ -73,13 +100,12 @@ struct scheduler_param {
       et   The event itself
 */
 
-void
-Event_worker_thread::print_warnings(THD *thd, Event_job_data *et)
-{
+void Event_worker_thread::print_warnings(THD *thd, Event_job_data *et) {
   const Sql_condition *err;
+  enum loglevel ll;
+
   DBUG_ENTER("evex_print_warnings");
-  if (thd->get_stmt_da()->cond_count() == 0)
-    DBUG_VOID_RETURN;
+  if (thd->get_stmt_da()->cond_count() == 0) DBUG_VOID_RETURN;
 
   char msg_buf[10 * STRING_BUFFER_USUAL_SIZE];
   char prefix_buf[5 * STRING_BUFFER_USUAL_SIZE];
@@ -87,42 +113,44 @@ Event_worker_thread::print_warnings(THD *thd, Event_job_data *et)
   prefix.length(0);
   prefix.append("Event Scheduler: [");
 
-  prefix.append(et->definer.str, et->definer.length, system_charset_info);
+  prefix.append(et->m_definer.str, et->m_definer.length, system_charset_info);
   prefix.append("][", 2);
-  prefix.append(et->dbname.str, et->dbname.length, system_charset_info);
+  prefix.append(et->m_schema_name.str, et->m_schema_name.length,
+                system_charset_info);
   prefix.append('.');
-  prefix.append(et->name.str, et->name.length, system_charset_info);
+  prefix.append(et->m_event_name.str, et->m_event_name.length,
+                system_charset_info);
   prefix.append("] ", 2);
 
-  Diagnostics_area::Sql_condition_iterator it=
-    thd->get_stmt_da()->sql_conditions();
-  while ((err= it++))
-  {
+  Diagnostics_area::Sql_condition_iterator it =
+      thd->get_stmt_da()->sql_conditions();
+  while ((err = it++)) {
     String err_msg(msg_buf, sizeof(msg_buf), system_charset_info);
     /* set it to 0 or we start adding at the end. That's the trick ;) */
     err_msg.length(0);
     err_msg.append(prefix);
-    err_msg.append(err->message_text(),
-                   err->message_octet_length(), system_charset_info);
-    switch (err->severity())
-    {
-    case Sql_condition::SL_ERROR:
-      sql_print_error("%*s", static_cast<int>(err_msg.length()), err_msg.c_ptr());
-      break;
-    case Sql_condition::SL_WARNING:
-      sql_print_warning("%*s", static_cast<int>(err_msg.length()), err_msg.c_ptr());
-      break;
-    case Sql_condition::SL_NOTE:
-      sql_print_information("%*s", static_cast<int>(err_msg.length()), err_msg.c_ptr());
-      break;
-    default:
-      assert(false);
+    err_msg.append(err->message_text(), err->message_octet_length(),
+                   system_charset_info);
+    switch (err->severity()) {
+      case Sql_condition::SL_ERROR:
+        ll = ERROR_LEVEL;
+        break;
+      case Sql_condition::SL_WARNING:
+        ll = WARNING_LEVEL;
+        break;
+      case Sql_condition::SL_NOTE:
+        ll = INFORMATION_LEVEL;
+        break;
+      default:
+        ll = ERROR_LEVEL;
+        DBUG_ASSERT(false);
     }
+    LogErr(ll, ER_EVENT_MESSAGE_STACK, static_cast<int>(err_msg.length()),
+           err_msg.c_ptr());
   }
   DBUG_VOID_RETURN;
 }
 
-
 /*
   Performs post initialization of structures in a new thread.
 
@@ -135,21 +163,18 @@ Event_worker_thread::print_warnings(THD *thd, Event_job_data *et)
 
 */
 
-bool
-post_init_event_thread(THD *thd)
-{
-  if (my_thread_init() || thd->store_globals())
-  {
-    return TRUE;
+bool post_init_event_thread(THD *thd) {
+  if (my_thread_init()) {
+    return true;
   }
+  thd->store_globals();
 
-  Global_THD_manager *thd_manager= Global_THD_manager::get_instance();
+  Global_THD_manager *thd_manager = Global_THD_manager::get_instance();
   thd_manager->add_thd(thd);
   thd_manager->inc_thread_running();
-  return FALSE;
+  return false;
 }
 
-
 /*
   Cleans up the THD and the threaded environment of the thread.
 
@@ -158,21 +183,18 @@ post_init_event_thread(THD *thd)
       thd  Thread
 */
 
-void
-deinit_event_thread(THD *thd)
-{
-  Global_THD_manager *thd_manager= Global_THD_manager::get_instance();
-  thd->proc_info= "Clearing";
+void deinit_event_thread(THD *thd) {
+  Global_THD_manager *thd_manager = Global_THD_manager::get_instance();
+
+  thd->proc_info = "Clearing";
   thd->get_protocol_classic()->end_net();
   DBUG_PRINT("exit", ("Event thread finishing"));
-
   thd->release_resources();
   thd_manager->remove_thd(thd);
   thd_manager->dec_thread_running();
   delete thd;
 }
 
-
 /*
   Performs pre- mysql_thread_create() initialisation of THD. Do this
   in the thread that will pass THD to the child thread. In the
@@ -187,19 +209,17 @@ deinit_event_thread(THD *thd)
     2. thd->net is initted with NULL - no communication.
 */
 
-void
-pre_init_event_thread(THD* thd)
-{
+void pre_init_event_thread(THD *thd) {
   DBUG_ENTER("pre_init_event_thread");
   thd->security_context()->set_master_access(0);
-  thd->security_context()->set_db_access(0);
-  thd->security_context()->set_host_or_ip_ptr((char *) my_localhost,
+  thd->security_context()->cache_current_db_access(0);
+  thd->security_context()->set_host_or_ip_ptr((char *)my_localhost,
                                               strlen(my_localhost));
   thd->get_protocol_classic()->init_net(NULL);
   thd->security_context()->set_user_ptr(C_STRING_WITH_LEN("event_scheduler"));
-  thd->get_protocol_classic()->get_net()->read_timeout= slave_net_timeout;
-  thd->slave_thread= 0;
-  thd->variables.option_bits|= OPTION_AUTO_IS_NULL;
+  thd->get_protocol_classic()->get_net()->read_timeout = slave_net_timeout;
+  thd->slave_thread = 0;
+  thd->variables.option_bits |= OPTION_AUTO_IS_NULL;
   thd->get_protocol_classic()->set_client_capabilities(CLIENT_MULTI_RESULTS);
 
   thd->set_new_thread_id();
@@ -208,16 +228,15 @@ pre_init_event_thread(THD* thd)
     vio is NULL.
   */
 
-  thd->proc_info= "Initialized";
+  thd->proc_info = "Initialized";
   thd->set_time();
 
   /* Do not use user-supplied timeout value for system threads. */
-  thd->variables.lock_wait_timeout= LONG_TIMEOUT;
+  thd->variables.lock_wait_timeout = LONG_TIMEOUT;
 
   DBUG_VOID_RETURN;
 }
 
-
 /*
   Function that executes the scheduler,
 
@@ -229,30 +248,18 @@ pre_init_event_thread(THD* thd)
     0  OK
 */
 
-extern "C" void *event_scheduler_thread(void *arg)
-{
+extern "C" {
+static void *event_scheduler_thread(void *arg) {
   /* needs to be first for thread_stack */
-  THD *thd= ((struct scheduler_param *) arg)->thd;
-  Event_scheduler *scheduler= ((struct scheduler_param *) arg)->scheduler;
+  THD *thd = ((struct scheduler_param *)arg)->thd;
+  Event_scheduler *scheduler = ((struct scheduler_param *)arg)->scheduler;
   bool res;
 
-  thd->thread_stack= (char *)&thd;              // remember where our stack is
+  thd->thread_stack = (char *)&thd;  // remember where our stack is
 
   mysql_thread_set_psi_id(thd->thread_id());
 
-#ifdef HAVE_PSI_THREAD_INTERFACE
-  /* Update the thread instrumentation. */
-  PSI_THREAD_CALL(set_thread_account)
-    (thd->security_context()->user().str,
-     thd->security_context()->user().length,
-     thd->security_context()->host_or_ip().str,
-     thd->security_context()->host_or_ip().length);
-  PSI_THREAD_CALL(set_thread_command)(thd->get_command());
-  PSI_THREAD_CALL(set_thread_start_time)(thd->query_start());
-  PSI_THREAD_CALL(set_thread_start_time_usec)(thd->query_start_usec());
-#endif /* HAVE_PSI_THREAD_INTERFACE */
-
-  res= post_init_event_thread(thd);
+  res = post_init_event_thread(thd);
 
   DBUG_ENTER("event_scheduler_thread");
   my_claim(arg);
@@ -260,19 +267,17 @@ extern "C" void *event_scheduler_thread(void *arg)
   my_free(arg);
   if (!res)
     scheduler->run(thd);
-  else
-  {
-    thd->proc_info= "Clearing";
+  else {
+    thd->proc_info = "Clearing";
     thd->get_protocol_classic()->end_net();
     delete thd;
   }
 
-  DBUG_LEAVE;                               // Against gcc warnings
+  DBUG_LEAVE;  // Against gcc warnings
   my_thread_end();
   return 0;
 }
 
-
 /**
   Function that executes an event in a child thread. Setups the
   environment for the event execution and cleans after that.
@@ -285,38 +290,25 @@ extern "C" void *event_scheduler_thread(void *arg)
     0  OK
 */
 
-extern "C" void *event_worker_thread(void *arg)
-{
+static void *event_worker_thread(void *arg) {
   THD *thd;
-  Event_queue_element_for_exec *event= (Event_queue_element_for_exec *)arg;
+  Event_queue_element_for_exec *event = (Event_queue_element_for_exec *)arg;
 
   event->claim_memory_ownership();
 
-  thd= event->thd;
+  thd = event->thd;
 
   thd->claim_memory_ownership();
 
   mysql_thread_set_psi_id(thd->thread_id());
 
-#ifdef HAVE_PSI_THREAD_INTERFACE
-  /* Update the thread instrumentation. */
-  PSI_THREAD_CALL(set_thread_account)
-    (thd->security_context()->user().str,
-     thd->security_context()->user().length,
-     thd->security_context()->host_or_ip().str,
-     thd->security_context()->host_or_ip().length);
-  PSI_THREAD_CALL(set_thread_command)(thd->get_command());
-  PSI_THREAD_CALL(set_thread_start_time)(thd->query_start());
-  PSI_THREAD_CALL(set_thread_start_time_usec)(thd->query_start_usec());
-#endif /* HAVE_PSI_THREAD_INTERFACE */
-
   Event_worker_thread worker_thread;
   worker_thread.run(thd, event);
 
   my_thread_end();
-  return 0;                                     // Can't return anything here
+  return 0;  // Can't return anything here
 }
-
+}  // extern "C"
 
 /**
   Function that executes an event in a child thread. Setups the
@@ -328,63 +320,84 @@ extern "C" void *event_worker_thread(void *arg)
       event  The Event_queue_element_for_exec object to be processed
 */
 
-void
-Event_worker_thread::run(THD *thd, Event_queue_element_for_exec *event)
-{
+void Event_worker_thread::run(THD *thd, Event_queue_element_for_exec *event) {
   /* needs to be first for thread_stack */
   char my_stack;
   Event_job_data job_data;
   bool res;
 
-  assert(thd->m_digest == NULL);
+  DBUG_ASSERT(thd->m_digest == NULL);
+
+  thd->thread_stack = &my_stack;  // remember where our stack is
+  res = post_init_event_thread(thd);
+
+  DBUG_ENTER("Event_worker_thread::run");
+  DBUG_PRINT("info", ("Time is %ld, THD: %p", (long)my_time(0), thd));
+
+  if (res) {
+    delete event;
+    deinit_event_thread(thd);
+    DBUG_VOID_RETURN;
+  }
 
 #ifdef HAVE_PSI_STATEMENT_INTERFACE
   PSI_statement_locker_state state;
-  assert(thd->m_statement_psi == NULL);
-  thd->m_statement_psi= MYSQL_START_STATEMENT(& state,
-                                              event->get_psi_info()->m_key,
-                                              event->dbname.str,
-                                              event->dbname.length,
-                                              thd->charset(), NULL);
+  DBUG_ASSERT(thd->m_statement_psi == NULL);
+  thd->m_statement_psi = MYSQL_START_STATEMENT(
+      &state, event->get_psi_info()->m_key, event->dbname.str,
+      event->dbname.length, thd->charset(), NULL);
 #endif
-
-  thd->thread_stack= &my_stack;                // remember where our stack is
-  res= post_init_event_thread(thd);
-
-  DBUG_ENTER("Event_worker_thread::run");
-  DBUG_PRINT("info", ("Time is %ld, THD: 0x%lx", (long) my_time(0), (long) thd));
-
-  if (res)
-    goto end;
-
-  if ((res= db_repository->load_named_event(thd, event->dbname, event->name,
-                                            &job_data)))
+  /*
+    We must make sure the schema is released and unlocked in the right
+    order. Fail if we are unable to get a meta data lock on the schema
+    name. Separate scope so that the Schema_MDL_locker dtor is run before
+    thd is deleted.
+  */
   {
-    DBUG_PRINT("error", ("Got error from load_named_event"));
-    goto end;
-  }
+    dd::Schema_MDL_locker mdl_handler(thd);
+    if (mdl_handler.ensure_locked(event->dbname.str)) goto end;
 
-  thd->enable_slow_log= TRUE;
+    MDL_key mdl_key;
+    dd::Event::create_mdl_key(event->dbname.str, event->name.str, &mdl_key);
 
-  res= job_data.execute(thd, event->dropped);
+    MDL_request event_mdl_request;
+    MDL_REQUEST_INIT_BY_KEY(&event_mdl_request, &mdl_key, MDL_SHARED,
+                            MDL_EXPLICIT);
+    if (thd->mdl_context.acquire_lock(&event_mdl_request,
+                                      thd->variables.lock_wait_timeout)) {
+      DBUG_PRINT("error", ("Got error in getting MDL locks"));
+      goto end;
+    }
+
+    if ((res = Event_db_repository::load_named_event(thd, event->dbname,
+                                                     event->name, &job_data))) {
+      DBUG_PRINT("error", ("Got error from load_named_event"));
+      thd->mdl_context.release_lock(event_mdl_request.ticket);
+      goto end;
+    }
+    thd->mdl_context.release_lock(event_mdl_request.ticket);
+  }  // End scope so that schema metadata lock is released.
+
+  thd->enable_slow_log = true;
+
+  res = job_data.execute(thd, event->dropped);
 
   print_warnings(thd, &job_data);
 
   if (res)
-    sql_print_information("Event Scheduler: "
-                          "[%s].[%s.%s] event execution failed.",
-                          job_data.definer.str,
-                          job_data.dbname.str, job_data.name.str);
+    LogErr(INFORMATION_LEVEL, ER_EVENT_EXECUTION_FAILED, job_data.m_definer.str,
+           job_data.m_schema_name.str, job_data.m_event_name.str);
+
 end:
 #ifdef HAVE_PSI_STATEMENT_INTERFACE
   MYSQL_END_STATEMENT(thd->m_statement_psi, thd->get_stmt_da());
-  thd->m_statement_psi= NULL;
+  thd->m_statement_psi = NULL;
 #endif
 
-  assert(thd->m_digest == NULL);
+  DBUG_ASSERT(thd->m_digest == NULL);
 
-  DBUG_PRINT("info", ("Done with Event %s.%s", event->dbname.str,
-             event->name.str));
+  DBUG_PRINT("info",
+             ("Done with Event %s.%s", event->dbname.str, event->name.str));
 
   delete event;
   deinit_event_thread(thd);
@@ -392,34 +405,28 @@ end:
   DBUG_VOID_RETURN;
 }
 
-
 Event_scheduler::Event_scheduler(Event_queue *queue_arg)
-  :state(INITIALIZED),
-  scheduler_thd(NULL),
-  queue(queue_arg),
-  mutex_last_locked_at_line(0),
-  mutex_last_unlocked_at_line(0),
-  mutex_last_locked_in_func("n/a"),
-  mutex_last_unlocked_in_func("n/a"),
-  mutex_scheduler_data_locked(FALSE),
-  waiting_on_cond(FALSE),
-  started_events(0)
-{
+    : state(INITIALIZED),
+      scheduler_thd(NULL),
+      queue(queue_arg),
+      mutex_last_locked_at_line(0),
+      mutex_last_unlocked_at_line(0),
+      mutex_last_locked_in_func("n/a"),
+      mutex_last_unlocked_in_func("n/a"),
+      mutex_scheduler_data_locked(false),
+      waiting_on_cond(false),
+      started_events(0) {
   mysql_mutex_init(key_event_scheduler_LOCK_scheduler_state,
                    &LOCK_scheduler_state, MY_MUTEX_INIT_FAST);
   mysql_cond_init(key_event_scheduler_COND_state, &COND_state);
 }
 
-
-Event_scheduler::~Event_scheduler()
-{
-  stop();                                    /* does nothing if not running */
+Event_scheduler::~Event_scheduler() {
+  stop(); /* does nothing if not running */
   mysql_mutex_destroy(&LOCK_scheduler_state);
   mysql_cond_destroy(&COND_state);
 }
 
-
-
 /**
   Starts the scheduler (again). Creates a new THD and passes it to
   a forked thread. Does not wait for acknowledgement from the new
@@ -435,79 +442,73 @@ Event_scheduler::~Event_scheduler()
     @retval true  Error.
 */
 
-bool
-Event_scheduler::start(int *err_no)
-{
-  THD *new_thd= NULL;
-  bool ret= false;
+bool Event_scheduler::start(int *err_no) {
+  THD *new_thd = NULL;
+  bool ret = false;
   my_thread_handle th;
   struct scheduler_param *scheduler_param_value;
   ulong master_access;
   DBUG_ENTER("Event_scheduler::start");
 
   LOCK_DATA();
-  DBUG_PRINT("info", ("state before action %s", scheduler_states_names[state].str));
-  if (state > INITIALIZED)
-    goto end;
+  DBUG_PRINT("info",
+             ("state before action %s", scheduler_states_names[state].str));
+  if (state > INITIALIZED) goto end;
 
   DBUG_EXECUTE_IF("event_scheduler_thread_create_failure", {
-                  *err_no= 11;
-                  Events::opt_event_scheduler= Events::EVENTS_OFF;
-                  ret= true;
-                  goto end; });
+    *err_no = 11;
+    Events::opt_event_scheduler = Events::EVENTS_OFF;
+    ret = true;
+    goto end;
+  });
 
-  if (!(new_thd= new THD))
-  {
-    sql_print_error("Event Scheduler: Cannot initialize the scheduler thread");
-    ret= true;
+  if (!(new_thd = new THD)) {
+    LogErr(ERROR_LEVEL, ER_CANT_INIT_SCHEDULER_THREAD);
+    ret = true;
     goto end;
   }
   pre_init_event_thread(new_thd);
-  new_thd->system_thread= SYSTEM_THREAD_EVENT_SCHEDULER;
+  new_thd->system_thread = SYSTEM_THREAD_EVENT_SCHEDULER;
   new_thd->set_command(COM_DAEMON);
 
   /*
     We should run the event scheduler thread under the super-user privileges.
-    In particular, this is needed to be able to lock the mysql.event table
+    In particular, this is needed to be able to lock the mysql.events table
     for writing when the server is running in the read-only mode.
 
     Same goes for transaction access mode. Set it to read-write for this thd.
   */
-  master_access= new_thd->security_context()->master_access();
+  master_access = new_thd->security_context()->master_access();
   new_thd->security_context()->set_master_access(master_access | SUPER_ACL);
-  new_thd->variables.tx_read_only=
-           new_thd->variables.transaction_read_only= false;
-  new_thd->tx_read_only= false;
+  new_thd->variables.transaction_read_only = false;
+  new_thd->tx_read_only = false;
 
-  scheduler_param_value=
-    (struct scheduler_param *)my_malloc(key_memory_Event_scheduler_scheduler_param,
-                                        sizeof(struct scheduler_param), MYF(0));
-  scheduler_param_value->thd= new_thd;
-  scheduler_param_value->scheduler= this;
+  scheduler_param_value = (struct scheduler_param *)my_malloc(
+      key_memory_Event_scheduler_scheduler_param,
+      sizeof(struct scheduler_param), MYF(0));
+  scheduler_param_value->thd = new_thd;
+  scheduler_param_value->scheduler = this;
 
-  scheduler_thd= new_thd;
+  scheduler_thd = new_thd;
   DBUG_PRINT("info", ("Setting state go RUNNING"));
-  state= RUNNING;
-  DBUG_PRINT("info", ("Forking new thread for scheduler. THD: 0x%lx", (long) new_thd));
-  if ((*err_no= mysql_thread_create(key_thread_event_scheduler,
-                                    &th, &connection_attrib,
-                                    event_scheduler_thread,
-                                    (void*)scheduler_param_value)))
-  {
+  state = RUNNING;
+  DBUG_PRINT("info", ("Forking new thread for scheduler. THD: %p", new_thd));
+  if ((*err_no = mysql_thread_create(key_thread_event_scheduler, &th,
+                                     &connection_attrib, event_scheduler_thread,
+                                     (void *)scheduler_param_value))) {
     DBUG_PRINT("error", ("cannot create a new thread"));
-    sql_print_error("Event scheduler: Failed to start scheduler,"
-                    " Can not create thread for event scheduler (errno=%d)",
-                    *err_no);
+    LogErr(ERROR_LEVEL, ER_CANT_CREATE_SCHEDULER_THREAD, *err_no)
+        .os_errno(*err_no);
 
-    new_thd->proc_info= "Clearing";
+    new_thd->proc_info = "Clearing";
     new_thd->get_protocol_classic()->end_net();
 
-    state= INITIALIZED;
-    scheduler_thd= NULL;
+    state = INITIALIZED;
+    scheduler_thd = NULL;
     delete new_thd;
 
-    delete scheduler_param_value;
-    ret= true;
+    my_free(scheduler_param_value);
+    ret = true;
   }
 
 end:
@@ -515,7 +516,6 @@ end:
   DBUG_RETURN(ret);
 }
 
-
 /*
   The main loop of the scheduler.
 
@@ -524,57 +524,46 @@ end:
       thd  Thread
 
   RETURN VALUE
-    FALSE  OK
-    TRUE   Error (Serious error)
+    false  OK
+    true   Error (Serious error)
 */
 
-bool
-Event_scheduler::run(THD *thd)
-{
-  int res= FALSE;
+bool Event_scheduler::run(THD *thd) {
+  bool res = false;
   DBUG_ENTER("Event_scheduler::run");
 
-  sql_print_information("Event Scheduler: scheduler thread started with id %u",
-                        thd->thread_id());
+  LogErr(INFORMATION_LEVEL, ER_SCHEDULER_STARTED, thd->thread_id());
   /*
     Recalculate the values in the queue because there could have been stops
     in executions of the scheduler and some times could have passed by.
   */
   queue->recalculate_activation_times(thd);
 
-  while (is_running())
-  {
+  while (is_running()) {
     Event_queue_element_for_exec *event_name;
 
     /* Gets a minimized version */
-    if (queue->get_top_for_execution_if_time(thd, &event_name))
-    {
-      sql_print_information("Event Scheduler: "
-                            "Serious error during getting next "
-                            "event to execute. Stopping");
+    if (queue->get_top_for_execution_if_time(thd, &event_name)) {
+      LogErr(INFORMATION_LEVEL, ER_SCHEDULER_STOPPING_FAILED_TO_GET_EVENT);
       break;
     }
 
     DBUG_PRINT("info", ("get_top_for_execution_if_time returned "
-                        "event_name=0x%lx", (long) event_name));
-    if (event_name)
-    {
-      if ((res= execute_top(event_name)))
-        break;
-    }
-    else
-    {
-      assert(thd->killed);
+                        "event_name=%p",
+                        event_name));
+    if (event_name) {
+      if ((res = execute_top(event_name))) break;
+    } else {
+      DBUG_ASSERT(thd->killed);
       DBUG_PRINT("info", ("job_data is NULL, the thread was killed"));
     }
     DBUG_PRINT("info", ("state=%s", scheduler_states_names[state].str));
-    free_root(thd->mem_root, MYF(0));
   }
 
   LOCK_DATA();
   deinit_event_thread(thd);
-  scheduler_thd= NULL;
-  state= INITIALIZED;
+  scheduler_thd = NULL;
+  state = INITIALIZED;
   DBUG_PRINT("info", ("Broadcasting COND_state back to the stoppers"));
   mysql_cond_broadcast(&COND_state);
   UNLOCK_DATA();
@@ -582,7 +571,6 @@ Event_scheduler::run(THD *thd)
   DBUG_RETURN(res);
 }
 
-
 /*
   Creates a new THD instance and then forks a new thread, while passing
   the THD pointer and job_data to it.
@@ -591,25 +579,22 @@ Event_scheduler::run(THD *thd)
     Event_scheduler::execute_top()
 
   RETURN VALUE
-    FALSE  OK
-    TRUE   Error (Serious error)
+    false  OK
+    true   Error (Serious error)
 */
 
-bool
-Event_scheduler::execute_top(Event_queue_element_for_exec *event_name)
-{
+bool Event_scheduler::execute_top(Event_queue_element_for_exec *event_name) {
   THD *new_thd;
   my_thread_handle th;
-  int res= 0;
+  int res = 0;
   DBUG_ENTER("Event_scheduler::execute_top");
-  if (!(new_thd= new THD()))
-    goto error;
+  if (!(new_thd = new THD())) goto error;
 
   pre_init_event_thread(new_thd);
-  new_thd->system_thread= SYSTEM_THREAD_EVENT_WORKER;
-  event_name->thd= new_thd;
-  DBUG_PRINT("info", ("Event %s@%s ready for start",
-             event_name->dbname.str, event_name->name.str));
+  new_thd->system_thread = SYSTEM_THREAD_EVENT_WORKER;
+  event_name->thd = new_thd;
+  DBUG_PRINT("info", ("Event %s@%s ready for start", event_name->dbname.str,
+                      event_name->name.str));
 
   /*
     TODO: should use thread pool here, preferably with an upper limit
@@ -620,18 +605,16 @@ Event_scheduler::execute_top(Event_queue_element_for_exec *event_name)
     reasonable level.
   */
   /* Major failure */
-  if ((res= mysql_thread_create(key_thread_event_worker,
-                                &th, &connection_attrib, event_worker_thread,
-                                event_name)))
-  {
+  if ((res =
+           mysql_thread_create(key_thread_event_worker, &th, &connection_attrib,
+                               event_worker_thread, event_name))) {
     mysql_mutex_lock(&LOCK_global_system_variables);
-    Events::opt_event_scheduler= Events::EVENTS_OFF;
+    Events::opt_event_scheduler = Events::EVENTS_OFF;
     mysql_mutex_unlock(&LOCK_global_system_variables);
 
-    sql_print_error("Event_scheduler::execute_top: Can not create event worker"
-                    " thread (errno=%d). Stopping event scheduler", res);
+    LogErr(ERROR_LEVEL, ER_SCHEDULER_STOPPING_FAILED_TO_CREATE_WORKER, res);
 
-    new_thd->proc_info= "Clearing";
+    new_thd->proc_info = "Clearing";
     new_thd->get_protocol_classic()->end_net();
 
     goto error;
@@ -639,19 +622,17 @@ Event_scheduler::execute_top(Event_queue_element_for_exec *event_name)
 
   ++started_events;
 
-  DBUG_PRINT("info", ("Event is in THD: 0x%lx", (long) new_thd));
-  DBUG_RETURN(FALSE);
+  DBUG_PRINT("info", ("Event is in THD: %p", new_thd));
+  DBUG_RETURN(false);
 
 error:
   DBUG_PRINT("error", ("Event_scheduler::execute_top() res: %d", res));
-  if (new_thd)
-    delete new_thd;
+  if (new_thd) delete new_thd;
 
   delete event_name;
-  DBUG_RETURN(TRUE);
+  DBUG_RETURN(true);
 }
 
-
 /*
   Checks whether the state of the scheduler is RUNNING
 
@@ -659,20 +640,17 @@ error:
     Event_scheduler::is_running()
 
   RETURN VALUE
-    TRUE   RUNNING
-    FALSE  Not RUNNING
+    true   RUNNING
+    false  Not RUNNING
 */
 
-bool
-Event_scheduler::is_running()
-{
+bool Event_scheduler::is_running() {
   LOCK_DATA();
-  bool ret= (state == RUNNING);
+  bool ret = (state == RUNNING);
   UNLOCK_DATA();
   return ret;
 }
 
-
 /**
   Stops the scheduler (again). Waits for acknowledgement from the
   scheduler that it has stopped - synchronous stopping.
@@ -684,21 +662,19 @@ Event_scheduler::is_running()
     Event_scheduler::stop()
 
   RETURN VALUE
-    FALSE  OK
-    TRUE   Error (not reported)
+    false  OK
+    true   Error (not reported)
 */
 
-bool
-Event_scheduler::stop()
-{
-  THD *thd= current_thd;
+bool Event_scheduler::stop() {
+  THD *thd = current_thd;
   DBUG_ENTER("Event_scheduler::stop");
-  DBUG_PRINT("enter", ("thd: 0x%lx", (long) thd));
+  DBUG_PRINT("enter", ("thd: %p", thd));
 
   LOCK_DATA();
-  DBUG_PRINT("info", ("state before action %s", scheduler_states_names[state].str));
-  if (state != RUNNING)
-  {
+  DBUG_PRINT("info",
+             ("state before action %s", scheduler_states_names[state].str));
+  if (state != RUNNING) {
     /* Synchronously wait until the scheduler stops. */
     while (state != INITIALIZED)
       COND_STATE_WAIT(thd, NULL, &stage_waiting_for_scheduler_to_stop);
@@ -709,8 +685,8 @@ Event_scheduler::stop()
   do {
     DBUG_PRINT("info", ("Waiting for COND_started_or_stopped from "
                         "the scheduler thread.  Current value of state is %s . "
-                        "workers count=%d", scheduler_states_names[state].str,
-                        workers_count()));
+                        "workers count=%d",
+                        scheduler_states_names[state].str, workers_count()));
     /*
       NOTE: We don't use kill_one_thread() because it can't kill COM_DEAMON
       threads. In addition, kill_one_thread() requires THD but during shutdown
@@ -721,28 +697,25 @@ Event_scheduler::stop()
       usage.
     */
 
-    state= STOPPING;
-    DBUG_PRINT("info", ("Scheduler thread has id %u",
-                        scheduler_thd->thread_id()));
+    state = STOPPING;
+    DBUG_PRINT("info",
+               ("Scheduler thread has id %u", scheduler_thd->thread_id()));
     /* Lock from delete */
     mysql_mutex_lock(&scheduler_thd->LOCK_thd_data);
     /* This will wake up the thread if it waits on Queue's conditional */
-    sql_print_information("Event Scheduler: Killing the scheduler thread, "
-                          "thread id %u",
-                          scheduler_thd->thread_id());
+    LogErr(INFORMATION_LEVEL, ER_SCHEDULER_KILLING, scheduler_thd->thread_id());
     scheduler_thd->awake(THD::KILL_CONNECTION);
     mysql_mutex_unlock(&scheduler_thd->LOCK_thd_data);
 
     /* thd could be 0x0, when shutting down */
-    sql_print_information("Event Scheduler: "
-                          "Waiting for the scheduler thread to reply");
+    LogErr(INFORMATION_LEVEL, ER_SCHEDULER_WAITING);
     COND_STATE_WAIT(thd, NULL, &stage_waiting_for_scheduler_to_stop);
   } while (state == STOPPING);
   DBUG_PRINT("info", ("Scheduler thread has cleaned up. Set state to INIT"));
-  sql_print_information("Event Scheduler: Stopped");
+  LogErr(INFORMATION_LEVEL, ER_SCHEDULER_STOPPED);
 end:
   UNLOCK_DATA();
-  DBUG_RETURN(FALSE);
+  DBUG_RETURN(false);
 }
 
 /**
@@ -751,18 +724,16 @@ end:
   from global thread list.
 */
 
-class Is_worker : public Do_THD_Impl
-{
-public:
+class Is_worker : public Do_THD_Impl {
+ public:
   Is_worker() : m_count(0) {}
-  virtual void operator()(THD *thd)
-  {
-    if (thd->system_thread == SYSTEM_THREAD_EVENT_WORKER)
-      m_count++;
+  virtual void operator()(THD *thd) {
+    if (thd->system_thread == SYSTEM_THREAD_EVENT_WORKER) m_count++;
     return;
   }
   int get_count() { return m_count; }
-private:
+
+ private:
   int m_count;
 };
 
@@ -773,19 +744,16 @@ private:
     Event_scheduler::workers_count()
 */
 
-int
-Event_scheduler::workers_count()
-{
-  int count= 0;
+int Event_scheduler::workers_count() {
+  int count = 0;
   Is_worker is_worker;
   DBUG_ENTER("Event_scheduler::workers_count");
   Global_THD_manager::get_instance()->do_for_all_thd(&is_worker);
-  count= is_worker.get_count();
+  count = is_worker.get_count();
   DBUG_PRINT("exit", ("%d", count));
   DBUG_RETURN(count);
 }
 
-
 /*
   Auxiliary function for locking LOCK_scheduler_state. Used
   by the LOCK_DATA macro.
@@ -796,19 +764,16 @@ Event_scheduler::workers_count()
       line  On which line mutex lock is requested
 */
 
-void
-Event_scheduler::lock_data(const char *func, uint line)
-{
+void Event_scheduler::lock_data(const char *func, uint line) {
   DBUG_ENTER("Event_scheduler::lock_data");
   DBUG_PRINT("enter", ("func=%s line=%u", func, line));
   mysql_mutex_lock(&LOCK_scheduler_state);
-  mutex_last_locked_in_func= func;
-  mutex_last_locked_at_line= line;
-  mutex_scheduler_data_locked= TRUE;
+  mutex_last_locked_in_func = func;
+  mutex_last_locked_at_line = line;
+  mutex_scheduler_data_locked = true;
   DBUG_VOID_RETURN;
 }
 
-
 /*
   Auxiliary function for unlocking LOCK_scheduler_state. Used
   by the UNLOCK_DATA macro.
@@ -819,19 +784,16 @@ Event_scheduler::lock_data(const char *func, uint line)
       line  On which line mutex unlock is requested
 */
 
-void
-Event_scheduler::unlock_data(const char *func, uint line)
-{
+void Event_scheduler::unlock_data(const char *func, uint line) {
   DBUG_ENTER("Event_scheduler::unlock_data");
   DBUG_PRINT("enter", ("func=%s line=%u", func, line));
-  mutex_last_unlocked_at_line= line;
-  mutex_scheduler_data_locked= FALSE;
-  mutex_last_unlocked_in_func= func;
+  mutex_last_unlocked_at_line = line;
+  mutex_scheduler_data_locked = false;
+  mutex_last_unlocked_in_func = func;
   mysql_mutex_unlock(&LOCK_scheduler_state);
   DBUG_VOID_RETURN;
 }
 
-
 /*
   Wrapper for mysql_cond_wait/timedwait
 
@@ -844,26 +806,25 @@ Event_scheduler::unlock_data(const char *func, uint line)
       line    On which line cond_wait is requested
 */
 
-void
-Event_scheduler::cond_wait(THD *thd, struct timespec *abstime, const PSI_stage_info *stage,
-                           const char *src_func, const char *src_file, uint src_line)
-{
+void Event_scheduler::cond_wait(THD *thd, struct timespec *abstime,
+                                const PSI_stage_info *stage,
+                                const char *src_func, const char *src_file,
+                                uint src_line) {
   DBUG_ENTER("Event_scheduler::cond_wait");
-  waiting_on_cond= TRUE;
-  mutex_last_unlocked_at_line= src_line;
-  mutex_scheduler_data_locked= FALSE;
-  mutex_last_unlocked_in_func= src_func;
+  waiting_on_cond = true;
+  mutex_last_unlocked_at_line = src_line;
+  mutex_scheduler_data_locked = false;
+  mutex_last_unlocked_in_func = src_func;
   if (thd)
-    thd->enter_cond(&COND_state, &LOCK_scheduler_state, stage,
-                    NULL, src_func, src_file, src_line);
+    thd->enter_cond(&COND_state, &LOCK_scheduler_state, stage, NULL, src_func,
+                    src_file, src_line);
 
-  DBUG_PRINT("info", ("mysql_cond_%swait", abstime? "timed":""));
+  DBUG_PRINT("info", ("mysql_cond_%swait", abstime ? "timed" : ""));
   if (!abstime)
     mysql_cond_wait(&COND_state, &LOCK_scheduler_state);
   else
     mysql_cond_timedwait(&COND_state, &LOCK_scheduler_state, abstime);
-  if (thd)
-  {
+  if (thd) {
     /*
       Need to unlock before exit_cond, so we need to relock.
       Not the best thing to do but we need to obey cond_wait()
@@ -872,14 +833,13 @@ Event_scheduler::cond_wait(THD *thd, struct timespec *abstime, const PSI_stage_i
     thd->exit_cond(NULL, src_func, src_file, src_line);
     LOCK_DATA();
   }
-  mutex_last_locked_in_func= src_func;
-  mutex_last_locked_at_line= src_line;
-  mutex_scheduler_data_locked= TRUE;
-  waiting_on_cond= FALSE;
+  mutex_last_locked_in_func = src_func;
+  mutex_last_locked_at_line = src_line;
+  mutex_scheduler_data_locked = true;
+  waiting_on_cond = false;
   DBUG_VOID_RETURN;
 }
 
-
 /*
   Dumps the internal status of the scheduler
 
@@ -887,23 +847,21 @@ Event_scheduler::cond_wait(THD *thd, struct timespec *abstime, const PSI_stage_i
     Event_scheduler::dump_internal_status()
 */
 
-void
-Event_scheduler::dump_internal_status()
-{
+void Event_scheduler::dump_internal_status() {
   DBUG_ENTER("Event_scheduler::dump_internal_status");
 
   puts("");
   puts("Event scheduler status:");
   printf("State      : %s\n", scheduler_states_names[state].str);
-  printf("Thread id  : %u\n", scheduler_thd? scheduler_thd->thread_id() : 0);
+  printf("Thread id  : %u\n", scheduler_thd ? scheduler_thd->thread_id() : 0);
   printf("LLA        : %s:%u\n", mutex_last_locked_in_func,
-                                 mutex_last_locked_at_line);
+         mutex_last_locked_at_line);
   printf("LUA        : %s:%u\n", mutex_last_unlocked_in_func,
-                                 mutex_last_unlocked_at_line);
-  printf("WOC        : %s\n", waiting_on_cond? "YES":"NO");
+         mutex_last_unlocked_at_line);
+  printf("WOC        : %s\n", waiting_on_cond ? "YES" : "NO");
   printf("Workers    : %d\n", workers_count());
-  printf("Executed   : %lu\n", (ulong) started_events);
-  printf("Data locked: %s\n", mutex_scheduler_data_locked ? "YES":"NO");
+  printf("Executed   : %lu\n", (ulong)started_events);
+  printf("Data locked: %s\n", mutex_scheduler_data_locked ? "YES" : "NO");
 
   DBUG_VOID_RETURN;
 }
