@@ -426,12 +426,20 @@ int create_zip_dict(THD *thd, const char *name, ulong name_len,
         break;
       case HA_ERR_RECORD_FILE_FULL:
         error = ER_RECORD_FILE_FULL;
-        my_error(error, MYF(0), "compression_dictionary");
+        my_error(error, MYF(0), COMPRESSION_DICTIONARY_TABLE);
         break;
       case HA_ERR_TOO_MANY_CONCURRENT_TRXS:
         error = ER_TOO_MANY_CONCURRENT_TRXS;
         my_error(error, MYF(0));
         break;
+      case HA_ERR_TABLE_READONLY:
+        error = ER_OPEN_AS_READONLY;
+        my_error(error, MYF(0), COMPRESSION_DICTIONARY_TABLE);
+        break;
+      case HA_ERR_INNODB_FORCED_RECOVERY:
+        error = ER_INNODB_FORCED_RECOVERY;
+        my_error(error, MYF(0));
+        break;
       default:
         DBUG_ASSERT(0);
         error = ER_UNKNOWN_ERROR;
@@ -580,6 +588,14 @@ int drop_zip_dict(THD *thd, const char *name, ulong name_len, bool if_exists) {
       error = ER_TOO_MANY_CONCURRENT_TRXS;
       my_error(error, MYF(0));
       break;
+    case HA_ERR_TABLE_READONLY:
+      error = ER_OPEN_AS_READONLY;
+      my_error(error, MYF(0), COMPRESSION_DICTIONARY_TABLE);
+      break;
+    case HA_ERR_INNODB_FORCED_RECOVERY:
+      error = ER_INNODB_FORCED_RECOVERY;
+      my_error(error, MYF(0));
+      break;
     default:
       DBUG_ASSERT(0);
       error = ER_UNKNOWN_ERROR;
@@ -767,13 +783,16 @@ static bool cols_table_delete_low(TABLE *table, uint64 table_id,
     ret = table->file->ha_delete_row(table->record[0]);
   }
 
-  if (ret == 0) {
-    return (false);
-  } else {
-    DBUG_ASSERT(0);
-    int error = ER_UNKNOWN_ERROR;
-    my_error(error, MYF(0));
-    return (true);
+  switch (ret) {
+    case 0:
+      return (false);
+    case HA_ERR_INNODB_FORCED_RECOVERY:
+      my_error(ER_INNODB_FORCED_RECOVERY, MYF(0));
+      return (true);
+    default:
+      DBUG_ASSERT(0);
+      my_error(ER_UNKNOWN_ERROR, MYF(0));
+      return (true);
   }
 }
 
