@@ -49,6 +49,7 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #include "rem0types.h"
 #include "row0types.h"
 #include "sql/dd/object_id.h"
+#include "sql/dd/types/init_mode.h"  // dict_init_mode_t
 #include "sync0rw.h"
 #include "trx0types.h"
 #include "univ.i"
@@ -1795,12 +1796,16 @@ dberr_t dict_get_dictionary_info_by_id(ulint dict_id, char **name,
 This can happen if Percona Server is bootstrapped with
 --innodb-encrypt-tables=ON If yes or if srv_encrypt_tables is ON/FORCE, during
 upgrade, mysql.ibd should be encrpted.
-@param[in]  is_upgrade          true in upgrade mode
+In no upgrade scenario it fetches encryption flag from first page of mysql.ibd
+to check whether it is encrypted.
+@param[in]  dict_init_mode      initalization mode
 @param[in]  mysql_plugin_space  space_id of mysql/plugin table. Used only
                                 during upgrade
-@return true if encrypted, false if not encrypted */
-bool dict_detect_encryption(bool is_upgrade, space_id_t mysql_plugin_space);
-
+@param[out] do_encrypt          true if encrypted, false if not encrypted
+@return true if success, false if failure */
+bool dict_detect_encryption_of_mysql_ibd(dict_init_mode_t dict_init_mode,
+                                         space_id_t mysql_plugin_space,
+                                         bool &encrypt_mysql);
 #include "dict0dict.ic"
 
 #endif
