@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2021, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -663,30 +663,6 @@ void getTextTransporterError(QQQQ) {
     {TE_SIGNAL_LOST,"Send failed for unknown reason(signal lost)"},
     //TE_SEND_BUFFER_FULL = 0x16
     {TE_SEND_BUFFER_FULL,"The send buffer was full, but sleeping for a while solved"},
-    //TE_SCI_LINK_ERROR = 0x0017
-    {TE_SCI_LINK_ERROR,"There is no link from this node to the switch"},
-    //TE_SCI_UNABLE_TO_START_SEQUENCE = 0x18 | TE_DO_DISCONNECT
-    {TE_SCI_UNABLE_TO_START_SEQUENCE,"Could not start a sequence, because system resources are exumed or no sequence has been created"},
-    //TE_SCI_UNABLE_TO_REMOVE_SEQUENCE = 0x19 | TE_DO_DISCONNECT
-    {TE_SCI_UNABLE_TO_REMOVE_SEQUENCE,"Could not remove a sequence"},
-    //TE_SCI_UNABLE_TO_CREATE_SEQUENCE = 0x1a | TE_DO_DISCONNECT
-    {TE_SCI_UNABLE_TO_CREATE_SEQUENCE,"Could not create a sequence, because system resources are exempted. Must reboot"},
-    //TE_SCI_UNRECOVERABLE_DATA_TFX_ERROR = 0x1b | TE_DO_DISCONNECT
-    {TE_SCI_UNRECOVERABLE_DATA_TFX_ERROR,"Tried to send data on redundant link but failed"},
-    //TE_SCI_CANNOT_INIT_LOCALSEGMENT = 0x1c | TE_DO_DISCONNECT
-    {TE_SCI_CANNOT_INIT_LOCALSEGMENT,"Cannot initialize local segment"},
-    //TE_SCI_CANNOT_MAP_REMOTESEGMENT = 0x1d | TE_DO_DISCONNEC
-    {TE_SCI_CANNOT_MAP_REMOTESEGMENT,"Cannot map remote segment"},
-    //TE_SCI_UNABLE_TO_UNMAP_SEGMENT = 0x1e | TE_DO_DISCONNECT
-    {TE_SCI_UNABLE_TO_UNMAP_SEGMENT,"Cannot free the resources used by this segment (step 1)"},
-    //TE_SCI_UNABLE_TO_REMOVE_SEGMENT = 0x1f  | TE_DO_DISCONNEC
-    {TE_SCI_UNABLE_TO_REMOVE_SEGMENT,"Cannot free the resources used by this segment (step 2)"},
-    //TE_SCI_UNABLE_TO_DISCONNECT_SEGMENT = 0x20 | TE_DO_DISCONNECT
-    {TE_SCI_UNABLE_TO_DISCONNECT_SEGMENT,"Cannot disconnect from a remote segment"},
-    //TE_SHM_IPC_PERMANENT = 0x21
-    {TE_SHM_IPC_PERMANENT,"Shm ipc Permanent error"},
-    //TE_SCI_UNABLE_TO_CLOSE_CHANNEL = 0x22
-    {TE_SCI_UNABLE_TO_CLOSE_CHANNEL, "Unable to close the sci channel and the resources allocated"},
     //TE_UNSUPPORTED_BYTE_ORDER = 0x23 | TE_DO_DISCONNECT
     {TE_UNSUPPORTED_BYTE_ORDER, "Error found in message (unsupported byte order)"},
     //TE_COMPRESSED_UNSUPPORTED = 0x24 | TE_DO_DISCONNECT
@@ -769,6 +745,14 @@ void getTextStartLog(QQQQ) {
 		       theData[3],
 		       theData[4]);
 }
+void getTextLCPRestored(QQQQ) {
+  //-----------------------------------------------------------------------
+  // REPORT Node Start completed restore of LCP.
+  //-----------------------------------------------------------------------
+  BaseString::snprintf(m_text, m_text_len,
+           "Node Start completed restore of LCP id: %u",
+           theData[1]);
+}
 void getTextStartREDOLog(QQQQ) {
   BaseString::snprintf(m_text, m_text_len, 
 		       "Node: %d StartLog: [GCI Keep: %d LastCompleted: %d NewestRestorable: %d]",
@@ -854,6 +838,63 @@ void getTextEventBufferStatus(QQQQ) {
 		       theData[5], theData[4],
 		       theData[7], theData[6]);
 }
+
+
+/** Give the text for the reason enum
+ * ndb_logevent_event_buffer_status_report_reason defined ndb_logevent.h
+ */
+const char *ndb_logevent_eventBuff_status_reasons[] = {
+  "NO_REPORT",
+  "COMPLETELY_BUFFERING",
+  "PARTIALLY_DISCARDING",
+  "COMPLETELY_DISCARDING",
+  "PARTIALLY_BUFFERING",
+  "BUFFERED_EPOCHS_OVER_THRESHOLD",
+  "ENOUGH_FREE_EVENTBUFFER",
+  "LOW_FREE_EVENTBUFFER",
+};
+
+const char* getReason(Uint32 reason)
+{
+  if (reason < NDB_ARRAY_SIZE(ndb_logevent_eventBuff_status_reasons))
+    return ndb_logevent_eventBuff_status_reasons[reason];
+  return "UNKNOWN reason code";
+}
+
+void getTextEventBufferStatus2(QQQQ) {
+  unsigned used= theData[1], alloc= theData[2], max_= theData[3];
+  const char *used_unit, *alloc_unit, *max_unit;
+  convert_unit(used, used_unit);
+  convert_unit(alloc, alloc_unit);
+  convert_unit(max_, max_unit);
+
+  BaseString used_pct_txt;
+  if (alloc != 0)
+  {
+    used_pct_txt.assfmt("(%d%% of alloc)",
+             (Uint32)((((Uint64)theData[1])*100)/theData[2]));
+  }
+
+  BaseString allocd_pct_txt;
+  if (max_ != 0)
+  {
+    allocd_pct_txt.assfmt("(%d%% of max)",
+             (Uint32)((((Uint64)theData[2])*100)/theData[3]));
+  }
+
+  BaseString::snprintf(m_text, m_text_len,
+		       "Event buffer status (0x%x): used=%d%s%s alloc=%d%s%s "
+		       "max=%d%s%s latest_consumed_epoch=%u/%u "
+                       "latest_buffered_epoch=%u/%u "
+                       "report_reason=%s",
+		       theData[8], used, used_unit, used_pct_txt.c_str(),
+		       alloc, alloc_unit, allocd_pct_txt.c_str(),
+		       max_, max_unit, (max_ == 0) ? "(unlimited)" : "",
+		       theData[5], theData[4],
+		       theData[7], theData[6],
+                       getReason(theData[9]));
+}
+
 void getTextWarningEvent(QQQQ) {
   BaseString::snprintf(m_text, m_text_len, "%s", (char *)&theData[1]);
 }
@@ -1448,6 +1489,7 @@ const EventLoggerBase::EventRepLogLevelMatrix EventLoggerBase::matrix[] = {
   ROW(NDBStopCompleted,        LogLevel::llStartUp,     1, Logger::LL_INFO ),
   ROW(NDBStopForced,           LogLevel::llStartUp,     1, Logger::LL_ALERT ),
   ROW(NDBStopAborted,          LogLevel::llStartUp,     1, Logger::LL_INFO ),
+  ROW(LCPRestored,             LogLevel::llStartUp,     7, Logger::LL_INFO ),
   ROW(StartREDOLog,            LogLevel::llStartUp,     4, Logger::LL_INFO ),
   ROW(StartLog,                LogLevel::llStartUp,    10, Logger::LL_INFO ),
   ROW(UNDORecordsExecuted,     LogLevel::llStartUp,    15, Logger::LL_INFO ),
@@ -1507,6 +1549,7 @@ const EventLoggerBase::EventRepLogLevelMatrix EventLoggerBase::matrix[] = {
   ROW(CreateLogBytes,          LogLevel::llInfo,  11, Logger::LL_INFO ),
   ROW(InfoEvent,               LogLevel::llInfo,   2, Logger::LL_INFO ),
   ROW(EventBufferStatus,       LogLevel::llInfo,   7, Logger::LL_INFO ),
+  ROW(EventBufferStatus2,       LogLevel::llInfo,   7, Logger::LL_INFO ),
 
   //Single User
   ROW(SingleUser,              LogLevel::llInfo,   7, Logger::LL_INFO ),
