@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2014, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -29,6 +29,7 @@
 #include <ndb_limits.h>
 #include <pc.hpp>
 #include <SimulatedBlock.hpp>
+#include <RWPool.hpp>
 #include <DLHashTable.hpp>
 #include <IntrusiveList.hpp>
 #include <DataBuffer.hpp>
@@ -45,11 +46,16 @@
 #include <trigger_definitions.h>
 #include <SignalCounter.hpp>
 #include <KeyTable.hpp>
+#include <portlib/NdbTick.h>
 #endif
 
 
 #define JAM_FILE_ID 350
 
+#define TIME_TRACK_HISTOGRAM_RANGES 32
+#define TIME_TRACK_LOG_HISTOGRAM_RANGES 5
+#define TIME_TRACK_INITIAL_RANGE_VALUE 50
+
 #ifdef DBTC_C
 /*
  * 2.2 LOCAL SYMBOLS
@@ -284,7 +290,8 @@ public:
   
   enum IndexState {
     IS_BUILDING = 0,          // build in progress, start state at create
-    IS_ONLINE = 1             // ready to use
+    IS_ONLINE = 1,            // ready to use
+    IS_OFFLINE = 2            // not in use
   };
 
   /* Sub states of IndexOperation while waiting for TransId_AI
@@ -298,7 +305,6 @@ public:
     ITAS_WAIT_KEY_FAIL = 4     // Failed collecting key
   };
   
-
   /**--------------------------------------------------------------------------
    * LOCAL SYMBOLS PER 'SYMBOL-VALUED' VARIABLE
    *
@@ -338,6 +344,9 @@ public:
     2.3 RECORDS AND FILESIZES
     -------------------------
   */
+  typedef DataBuffer<11,ArrayPool<DataBufferSegment<11> > > AttributeBuffer;
+  typedef LocalDataBuffer<11,ArrayPool<DataBufferSegment<11> > > LocalAttributeBuffer;
+
   /* **************************************************************** */
   /* ---------------------------------------------------------------- */
   /* ------------------- TRIGGER AND INDEX DATA --------------------- */
@@ -400,23 +409,25 @@ public:
     }
   };
   typedef Ptr<TcDefinedTriggerData> DefinedTriggerPtr;
+  typedef ArrayPool<TcDefinedTriggerData> TcDefinedTriggerData_pool;
+  typedef DLList<TcDefinedTriggerData_pool> TcDefinedTriggerData_list;
   
   /**
    * Pool of trigger data record
    */
-  ArrayPool<TcDefinedTriggerData> c_theDefinedTriggerPool;
+  TcDefinedTriggerData_pool c_theDefinedTriggerPool;
   RSS_AP_SNAPSHOT(c_theDefinedTriggerPool);
 
   /**
    * The list of active triggers
    */  
-  DLList<TcDefinedTriggerData> c_theDefinedTriggers;
+  TcDefinedTriggerData_list c_theDefinedTriggers;
 
-  typedef DataBuffer<11> AttributeBuffer;
- 
   AttributeBuffer::DataBufferPool c_theAttributeBufferPool;
 
-  typedef DataBuffer<5> CommitAckMarkerBuffer;
+  typedef ArrayPool<DataBufferSegment<5> > CommitAckMarkerBuffer_pool;
+  typedef DataBuffer<5,CommitAckMarkerBuffer_pool> CommitAckMarkerBuffer;
+  typedef LocalDataBuffer<5,CommitAckMarkerBuffer_pool> LocalCommitAckMarkerBuffer;
 
   CommitAckMarkerBuffer::DataBufferPool c_theCommitAckMarkerBufferPool;
 
@@ -511,18 +522,25 @@ public:
     }
   };
   typedef Ptr<TcFiredTriggerData> FiredTriggerPtr;
+  typedef ArrayPool<TcFiredTriggerData> TcFiredTriggerData_pool;
+  typedef DLFifoList<TcFiredTriggerData_pool> TcFiredTriggerData_fifo;
+  typedef LocalDLFifoList<TcFiredTriggerData_pool> Local_TcFiredTriggerData_fifo;
+  typedef DLHashTable<TcFiredTriggerData_pool> TcFiredTriggerData_hash;
   
   /**
    * Pool of trigger data record
    */
-  ArrayPool<TcFiredTriggerData> c_theFiredTriggerPool;
-  DLHashTable<TcFiredTriggerData> c_firedTriggerHash;
+  TcFiredTriggerData_pool c_theFiredTriggerPool;
+  TcFiredTriggerData_hash c_firedTriggerHash;
   AttributeBuffer::DataBufferPool c_theTriggerAttrInfoPool;
   RSS_AP_SNAPSHOT(c_theFiredTriggerPool);
 
   Uint32 c_maxNumberOfDefinedTriggers;
   Uint32 c_maxNumberOfFiredTriggers;
 
+  // Max number of outstanding FireTrigRequests per transaction
+  static const Uint32 MaxOutstandingFireTrigReqPerTrans = 32;
+
   struct AttrInfoRecord {
     /**
      * Pre-allocated AttrInfo signal
@@ -552,7 +570,9 @@ public:
   /* WHEN THE INDEX IS DROPPED.               */
   /* **************************************** */
   struct TcIndexData {
-    TcIndexData() {}
+    TcIndexData() :
+      indexState(IS_OFFLINE)
+    {}
 
     /**
      *  IndexState
@@ -593,17 +613,19 @@ public:
   };
   
   typedef Ptr<TcIndexData> TcIndexDataPtr;
-  
+  typedef ArrayPool<TcIndexData> TcIndexData_pool;
+  typedef DLList<TcIndexData_pool> TcIndexData_list;
+
   /**
    * Pool of index data record
    */
-  ArrayPool<TcIndexData> c_theIndexPool;
+  TcIndexData_pool c_theIndexPool;
   RSS_AP_SNAPSHOT(c_theIndexPool);
   
   /**
    * The list of defined indexes
    */  
-  DLList<TcIndexData> c_theIndexes;
+  TcIndexData_list c_theIndexes;
   UintR c_maxNumberOfIndexes;
 
   struct TcIndexOperation {
@@ -656,11 +678,14 @@ public:
   };
   
   typedef Ptr<TcIndexOperation> TcIndexOperationPtr;
-  
+  typedef ArrayPool<TcIndexOperation> TcIndexOperation_pool;
+  typedef SLList<TcIndexOperation_pool> TcIndexOperation_sllist;
+  typedef DLList<TcIndexOperation_pool> TcIndexOperation_dllist;
+
   /**
    * Pool of index data record
    */
-  ArrayPool<TcIndexOperation> c_theIndexOperationPool;
+  TcIndexOperation_pool c_theIndexOperationPool;
   RSS_AP_SNAPSHOT(c_theIndexOperationPool);
 
   UintR c_maxNumberOfIndexOperations;   
@@ -703,8 +728,8 @@ public:
     }
   };
 
-  typedef RecordPool<TcFKData, RWPool> FK_pool;
-  typedef KeyTableImpl<FK_pool, TcFKData> FK_hash;
+  typedef RecordPool<RWPool<TcFKData> > FK_pool;
+  typedef KeyTable<FK_pool> FK_hash;
 
   FK_pool c_fk_pool;
   FK_hash c_fk_hash;
@@ -744,16 +769,26 @@ public:
   UintR* c_apiConTimer;
   UintR* c_apiConTimer_line;
 
-  // max cascading scans (FK child scans) per transaction
+  /**
+   * Limit the resource (signal/job buffer) usage of a transaction
+   * by limiting :
+   * - max cascading scans (FK child scans) and
+   * - trigger operations.
+   * An FK child scan is executed alone exclusively.
+   */
   static const Uint8 MaxCascadingScansPerTransaction = 1;
+  static const Uint32 MaxExecutingTriggerOpsPerTrans = 32;
 
   struct ApiConnectRecord {
-    ApiConnectRecord(ArrayPool<TcFiredTriggerData> & firedTriggerPool,
-		     ArrayPool<TcIndexOperation> & seizedIndexOpPool):
+    ApiConnectRecord(TcFiredTriggerData_pool & firedTriggerPool,
+                     TcIndexOperation_pool & seizedIndexOpPool):
+      nextApiConnect(RNIL),
       m_special_op_flags(0),
       theFiredTriggers(firedTriggerPool),
       theSeizedIndexOperations(seizedIndexOpPool) 
-    {}
+    {
+      NdbTick_Invalidate(&m_start_ticks);
+    }
     
     //---------------------------------------------------
     // First 16 byte cache line. Hot variables.
@@ -830,6 +865,7 @@ public:
       TF_DEFERRED_UK_TRIGGERS = 32, // trans has deferred UK triggers
       TF_DEFERRED_FK_TRIGGERS = 64, // trans has deferred FK triggers
       TF_DISABLE_FK_CONSTRAINTS = 128,
+      TF_LATE_COMMIT = 256, // Wait sending apiCommit until complete phase done
 
       TF_END = 0
     };
@@ -843,7 +879,15 @@ public:
     Uint8 tckeyrec; // Changed from R
 
     Uint8 tcindxrec;
-    Uint8 apiFailState; // Changed R
+
+    enum ApiFailStates
+    {
+      AFS_API_OK = 0,
+      AFS_API_FAILED = 1,
+      AFS_API_DISCONNECTED = 2
+    };
+    Uint8 apiFailState;
+
     Uint8 timeOutCounter;
     Uint8 singleUserMode;
     
@@ -879,9 +923,19 @@ public:
     /**
      * The list of fired triggers
      */  
-    DLFifoList<TcFiredTriggerData> theFiredTriggers;
-    
-    
+    TcFiredTriggerData_fifo theFiredTriggers;
+
+    // Count the outstanding FIRE_TRIG_REQs of a transaction.
+    // Limit it in order to avoid job buffer overload
+    Uint32 m_outstanding_fire_trig_req;
+
+    // First and last indices of the local tc connect pointers that will
+    // be used to send fire trigger reqs when resumed in execFireTrigConf
+    // or in execCONTINUEB
+    UintR m_firstTcConnectPtrI_FT;
+    UintR m_lastTcConnectPtrI_FT;
+
+
     // Index data
     
     UintR noIndexOp;     // No outstanding index ops
@@ -892,11 +946,13 @@ public:
     Uint32 errorData;
     UintR attrInfoLen;
     Uint32 immediateTriggerId;  // Id of trigger op being fired NOW
+    Uint32 firedFragId;
     
     UintR accumulatingIndexOp;
     UintR executingIndexOp;
     UintR tcIndxSendArray[6];
-    DLList<TcIndexOperation> theSeizedIndexOperations;
+    NDB_TICKS m_start_ticks;
+    TcIndexOperation_dllist theSeizedIndexOperations;
 
 #ifdef ERROR_INSERT
     Uint32 continueBCount;  // ERROR_INSERT 8082
@@ -908,8 +964,54 @@ public:
         apiConnectstate == CS_WAIT_FIRE_TRIG_REQ ;
     }
 
-    // number of on-going cascading scans (FK child scans)
+    // number of on-going cascading scans (FK child scans) at a transaction.
     Uint8 cascading_scans_count;
+
+    // Number of on-going trigger operations at a transaction
+    // Limit them in order to avoid the transaction
+    // overloading node resources (signal/job buffers).
+    Uint32 m_executing_trigger_ops;
+
+    // Trigger execution loop active
+    bool m_inExecuteTriggers;
+    /**
+     * ExecTriggersGuard
+     *
+     * Used to avoid recursive calls of executeTriggers
+     */
+    class ExecTriggersGuard
+    {
+      ApiConnectRecord* m_recPtr;
+    public:
+      ExecTriggersGuard(ApiConnectRecord* recPtr)
+      {
+        if (recPtr->m_inExecuteTriggers)
+        {
+          m_recPtr = NULL;
+        }
+        else
+        {
+          m_recPtr = recPtr;
+          m_recPtr->m_inExecuteTriggers = true;
+        }
+      }
+
+      ~ExecTriggersGuard()
+      {
+        if (m_recPtr)
+        {
+          assert(m_recPtr->m_inExecuteTriggers == true);
+          m_recPtr->m_inExecuteTriggers = false;
+        }
+      }
+
+      bool canExecNow() const
+      {
+        assert(m_recPtr == NULL ||
+               m_recPtr->m_inExecuteTriggers);
+        return (m_recPtr != NULL);
+      }
+    };
   };
   
   typedef Ptr<ApiConnectRecord> ApiConnectRecordPtr;
@@ -932,6 +1034,10 @@ public:
   /*       TC_CONNECT RECORD ALIGNED TO BE 128 BYTES                   */
   /*******************************************************************>*/
   struct TcConnectRecord {
+    TcConnectRecord()
+    {
+      NdbTick_Invalidate(&m_start_ticks);
+    }
     //---------------------------------------------------
     // First 16 byte cache line. Those variables are only
     // used in error cases.
@@ -968,15 +1074,16 @@ public:
     enum SpecialOpFlags {
       SOF_NORMAL = 0,
       SOF_INDEX_TABLE_READ = 1,       // Read index table
-      SOF_REORG_TRIGGER_BASE = 4,
-      SOF_REORG_TRIGGER = 4 | 16,     // A reorg trigger
+      SOF_REORG_TRIGGER = 4,          // A reorg trigger
       SOF_REORG_MOVING = 8,           // A record that should be moved
       SOF_TRIGGER = 16,               // A trigger
       SOF_REORG_COPY = 32,
       SOF_REORG_DELETE = 64,
       SOF_DEFERRED_UK_TRIGGER = 128,  // Op has deferred trigger
       SOF_DEFERRED_FK_TRIGGER = 256,
-      SOF_FK_READ_COMMITTED = 512     // reply to TC even for dirty read
+      SOF_FK_READ_COMMITTED = 512,    // reply to TC even for dirty read
+      SOF_FULLY_REPLICATED_TRIGGER = 1024,
+      SOF_UTIL_FLAG = 2048            // Sender to TC is DBUTIL (higher prio)
     };
     
     static inline bool isIndexOp(Uint16 flags) {
@@ -1004,7 +1111,7 @@ public:
     /**
      * The list of pending fired triggers
      */
-    DLFifoList<TcFiredTriggerData>::Head thePendingTriggers;
+    TcFiredTriggerData_fifo::Head thePendingTriggers;
 
     UintR triggeringOperation;  // Which operation was "cause" of this op
     
@@ -1015,6 +1122,7 @@ public:
       Uint32 attrInfoLen;
       Uint32 triggerErrorCode;
     };
+    NDB_TICKS m_start_ticks;
   };
   
   friend struct TcConnectRecord;
@@ -1024,7 +1132,7 @@ public:
   // ********************** CACHE RECORD **************************************
   //---------------------------------------------------------------------------
   // This record is used between reception of TCKEYREQ and sending of LQHKEYREQ
-  // It is separatedso as to improve the cache hit rate and also to minimise 
+  // It is separated so as to improve the cache hit rate and also to minimise 
   // the necessary memory storage in NDB Cluster.
   //---------------------------------------------------------------------------
 
@@ -1066,7 +1174,8 @@ public:
        * EXECUTION MODE OF OPERATION                    
        * 0 = NORMAL EXECUTION, 1 = INTERPRETED EXECUTION
        */
-      Uint8  opExec;     
+      Uint8  opExec;
+      Uint8  m_read_committed_base;
     
       /* Use of Long signals */
       Uint8  isLongTcKeyReq;   /* Incoming TcKeyReq used long signal */
@@ -1095,6 +1204,8 @@ public:
     LqhTransState lqhTransStatus;
     bool  inPackedList;
 
+    Uint32 m_location_domain_id;
+
     enum NodeFailBits
     {
       NF_TAKEOVER          = 0x1,
@@ -1105,7 +1216,28 @@ public:
     };
     Uint32 m_nf_bits;
     NdbNodeBitmask m_lqh_trans_conf;
+    /**
+     * Indicator if any history to track yet
+     *
+     * Tracking scan and scan errors (API node)
+     * Tracking read key, write key and index key operations
+     * (API node and primary DB node)
+     * Tracking scan frag and scan frag errors (API node)
+     * Tracking transactions (API node)
+     */
+    Uint32 time_tracked;
+    Uint64 time_track_scan_histogram[TIME_TRACK_HISTOGRAM_RANGES];
+    Uint64 time_track_scan_error_histogram[TIME_TRACK_HISTOGRAM_RANGES];
+    Uint64 time_track_read_key_histogram[TIME_TRACK_HISTOGRAM_RANGES];
+    Uint64 time_track_write_key_histogram[TIME_TRACK_HISTOGRAM_RANGES];
+    Uint64 time_track_index_key_histogram[TIME_TRACK_HISTOGRAM_RANGES];
+    Uint64 time_track_key_error_histogram[TIME_TRACK_HISTOGRAM_RANGES];
+    Uint64 time_track_scan_frag_histogram[TIME_TRACK_HISTOGRAM_RANGES];
+    Uint64 time_track_scan_frag_error_histogram[TIME_TRACK_HISTOGRAM_RANGES];
+    Uint64 time_track_transaction_histogram[TIME_TRACK_HISTOGRAM_RANGES];
+    Uint64 time_track_transaction_error_histogram[TIME_TRACK_HISTOGRAM_RANGES];
   };
+  Uint32 m_my_location_domain_id;
   
   typedef Ptr<HostRecord> HostRecordPtr;
   
@@ -1128,6 +1260,9 @@ public:
       TR_STORED_TABLE = 1 << 2,
       TR_PREPARED     = 1 << 3
       ,TR_USER_DEFINED_PARTITIONING = 1 << 4
+      ,TR_READ_BACKUP = (1 << 5)
+      ,TR_FULLY_REPLICATED = (1<<6)
+      ,TR_DELAY_COMMIT = (1 << 7)
     };
     Uint8 get_enabled()     const { return (m_flags & TR_ENABLED)      != 0; }
     Uint8 get_dropping()    const { return (m_flags & TR_DROPPING)     != 0; }
@@ -1156,7 +1291,9 @@ public:
     bool checkTable(Uint32 schemaVersion) const {
       return !get_dropping() &&
 	((/** normal transaction path */
-          get_enabled() && table_version_major(schemaVersion) == table_version_major(currentSchemaVersion)) 
+          get_enabled() &&
+          table_version_major(schemaVersion) ==
+          table_version_major(currentSchemaVersion)) 
          ||
          (/** 
            * unique index is relaxed for DbUtil and transactions ongoing
@@ -1170,6 +1307,34 @@ public:
   };
   typedef Ptr<TableRecord> TableRecordPtr;
 
+  /**
+   * Specify the location of a fragment. The 'blockRef' is either
+   * the specific LQH where the fragId resides, or the SPJ block
+   * responsible for scaning this fragment, if 'viaSPJ'.
+   */
+  struct ScanFragLocationRec
+  {
+    Uint32 blockRef;
+    Uint32 fragId;
+
+    /**
+     * Next ptr (used in pool/list)
+     */
+    union {
+      Uint32 nextPool;
+      Uint32 nextList;
+    };
+
+    Uint32 m_magic;    //Needed by RWPool
+  };
+
+  typedef Ptr<ScanFragLocationRec> ScanFragLocationPtr;
+  typedef RecordPool<RWPool<ScanFragLocationRec> > ScanFragLocation_pool;
+  typedef SLFifoList<ScanFragLocation_pool> ScanFragLocation_list;
+  typedef LocalSLFifoList<ScanFragLocation_pool> Local_ScanFragLocation_list;
+
+  ScanFragLocation_pool m_fragLocationPool;
+
   /**
    * There is max 16 ScanFragRec's for 
    * each scan started in TC. Each ScanFragRec is used by
@@ -1182,19 +1347,20 @@ public:
       lqhBlockref = 0;
       scanFragState = COMPLETED;
       scanRec = RNIL;
+      NdbTick_Invalidate(&m_start_ticks);
     }
     /**
      * ScanFragState      
      *  WAIT_GET_PRIMCONF : Waiting for DIGETPRIMCONF when starting a new 
-     *   fragment scan
+     *   fragment scan (Obsolete; Checked for, but never set)
      *  LQH_ACTIVE : The scan process has sent a command to LQH and is
      *   waiting for the response
      *  LQH_ACTIVE_CLOSE : The scan process has sent close to LQH and is
-     *   waiting for the response
+     *   waiting for the response (Unused)
      *  DELIVERED : The result have been delivered, this scan frag process 
      *   are waiting for a SCAN_NEXTREQ to tell us to continue scanning
      *  RETURNING_FROM_DELIVERY : SCAN_NEXTREQ received and continuing scan
-     *   soon 
+     *   soon (Unused)
      *  QUEUED_FOR_DELIVERY : Result queued in TC and waiting for delivery
      *   to API
      *  COMPLETED : The fragment scan processes has completed and finally
@@ -1211,8 +1377,8 @@ public:
     // Timer for checking timeout of this fragment scan
     Uint32  scanFragTimer;
 
-    // Id of the current scanned fragment
-    Uint32 scanFragId;
+    // Fragment id as reported back by DIGETNODESREQ
+    Uint32 lqhScanFragId;
 
     // Blockreference of LQH 
     BlockReference lqhBlockref;
@@ -1245,10 +1411,14 @@ public:
       Uint32 nextList;
     };
     Uint32 prevList;
+    NDB_TICKS m_start_ticks;
   };
   
   typedef Ptr<ScanFragRec> ScanFragRecPtr;
-  typedef LocalDLList<ScanFragRec> ScanFragList;
+  typedef UnsafeArrayPool<ScanFragRec> ScanFragRec_pool;
+  typedef SLList<ScanFragRec_pool> ScanFragRec_sllist;
+  typedef DLList<ScanFragRec_pool> ScanFragRec_dllist;
+  typedef LocalDLList<ScanFragRec_pool> Local_ScanFragRec_dllist;
 
   /**
    * Each scan allocates one ScanRecord to store information 
@@ -1256,7 +1426,10 @@ public:
    *
    */
   struct ScanRecord {
-    ScanRecord() {}
+    ScanRecord()
+    {
+      NdbTick_Invalidate(&m_start_ticks);
+    }
     /** NOTE! This is the old comment for ScanState. - MASV
      *       STATE TRANSITIONS OF SCAN_STATE. SCAN_STATE IS THE STATE 
      *       VARIABLE OF THE RECEIVE AND DELIVERY PROCESS.
@@ -1325,10 +1498,13 @@ public:
     Uint32 scanKeyInfoPtr;
     Uint32 scanAttrInfoPtr;
 
-    DLList<ScanFragRec>::Head m_running_scan_frags;  // Currently in LQH
+    // List of fragment locations as reported by DIH
+    ScanFragLocation_list::Head m_fragLocations;
+
+    ScanFragRec_dllist::Head m_running_scan_frags;  // Currently in LQH
     union { Uint32 m_queued_count; Uint32 scanReceivedOperations; };
-    DLList<ScanFragRec>::Head m_queued_scan_frags;   // In TC !sent to API
-    DLList<ScanFragRec>::Head m_delivered_scan_frags;// Delivered to API
+    ScanFragRec_dllist::Head m_queued_scan_frags;   // In TC !sent to API
+    ScanFragRec_dllist::Head m_delivered_scan_frags;// Delivered to API
     
     // Id of the next fragment to be scanned. Used by scan fragment 
     // processes when they are ready for the next fragment
@@ -1382,12 +1558,15 @@ public:
      * Send opcount/total len as different words
      */
     bool m_4word_conf;
+    bool m_read_committed_base;
 
     /**
      *
      */
     bool m_scan_dist_key_flag;
     Uint32 m_scan_dist_key;
+    Uint32 m_read_any_node;
+    NDB_TICKS m_start_ticks;
   };
   typedef Ptr<ScanRecord> ScanRecordPtr;
   
@@ -1401,9 +1580,7 @@ public:
   /*       GCP RECORD ALIGNED TO BE 32 BYTES                                 */
   /*************************************************************************>*/
   struct GcpRecord {
-    Uint16 gcpUnused0;
     Uint16 gcpNomoretransRec;
-    UintR gcpUnused1[2];	/* p2c: Not used */
     UintR firstApiConnect;
     UintR lastApiConnect;
     UintR nextGcp;
@@ -1448,7 +1625,6 @@ private:
   void execSCAN_NEXTREQ(Signal* signal);
   void execSCAN_PROCREQ(Signal* signal);
   void execSCAN_PROCCONF(Signal* signal);
-  void execTAKE_OVERTCREQ(Signal* signal);
   void execTAKE_OVERTCCONF(Signal* signal);
   void execLQHKEYREF(Signal* signal);
   void execTRANSID_AI_R(Signal* signal);
@@ -1461,11 +1637,12 @@ private:
   void execCOMPLETED(Signal* signal);
   void execCOMMITTED(Signal* signal);
   void execDIGETNODESREF(Signal* signal);
-  void execDIH_SCAN_GET_NODES_REF(Signal* signal);
-  void execDIH_SCAN_GET_NODES_CONF(Signal* signal);
   void execDIVERIFYCONF(Signal* signal);
-  void execDIH_SCAN_TAB_REF(Signal* signal);
-  void execDIH_SCAN_TAB_CONF(Signal* signal);
+  void execDIH_SCAN_TAB_REF(Signal* signal,
+                            ScanRecordPtr scanptr);
+  void execDIH_SCAN_TAB_CONF(Signal* signal,
+                             ScanRecordPtr scanptr,
+                             TableRecordPtr tabPtr);
   void execGCP_NOMORETRANS(Signal* signal);
   void execLQHKEYCONF(Signal* signal);
   void execNDB_STTOR(Signal* signal);
@@ -1559,9 +1736,14 @@ private:
                          TcConnectRecord * const regTcPtr);
 
   void startSendFireTrigReq(Signal*, Ptr<ApiConnectRecord>);
-  void sendFireTrigReq(Signal*, Ptr<ApiConnectRecord>,
-                       Uint32 firstTcConnect, Uint32 lastTcConnect);
-  Uint32 sendFireTrigReqLqh(Signal*, Ptr<TcConnectRecord>, Uint32 pass);
+
+  void sendFireTrigReq(Signal*, Ptr<ApiConnectRecord>);
+
+  Uint32 sendFireTrigReqLqh(Signal*, Ptr<TcConnectRecord>, Uint32 pass,
+                            Ptr<ApiConnectRecord>);
+
+  void checkWaitFireTrigConfDone(Signal*,
+                                 Ptr<ApiConnectRecord>);
 
 /**
  * These use modulo 2 hashing, so these need to be a number which is 2^n.
@@ -1581,6 +1763,7 @@ private:
   void timeOutLoopStartFragLab(Signal* signal, Uint32 TscanConPtr);
   int  releaseAndAbort(Signal* signal);
 
+  void scan_for_read_backup(Signal *, Uint32, Uint32, Uint32);
   void releaseMarker(ApiConnectRecord * const regApiPtr);
 
   Uint32 get_transid_fail_bucket(Uint32 transid1);
@@ -1627,6 +1810,11 @@ private:
                          LqhTransConf::OperationStatus transStatus,
                          NodeId nodeId);
 
+  bool handleFailedApiConnection(Signal*,
+                                 Uint32 *TloopCount,
+                                 Uint32 TapiFailedNode,
+                                 bool apiNodeFailed);
+  void set_api_fail_state(Uint32 TapiFailedNode, bool apiNodeFailed);
   void handleApiFailState(Signal* signal, UintR anApiConnectptr);
   void handleFailedApiNode(Signal* signal,
                            UintR aFailedNode,
@@ -1635,18 +1823,15 @@ private:
   void initScanTcrec(Signal* signal);
   Uint32 initScanrec(ScanRecordPtr,  const class ScanTabReq*,
                      const UintR scanParallel,
-                     const UintR noOprecPerFrag,
                      const Uint32 apiPtr[]);
   void initScanfragrec(Signal* signal);
   void releaseScanResources(Signal*, ScanRecordPtr, bool not_started = false);
   ScanRecordPtr seizeScanrec(Signal* signal);
-  void startFragScansLab(Signal*, Uint32 tableId,
-                        SectionHandle&, Uint32 secOffs);
-  void startFragScanLab(Signal*, Uint32 tableId,
-                        const DihScanGetNodesConf::FragItem& fragConf);
 
-  void sendDihGetNodesReq(Signal*, ScanRecordPtr);
-  void sendScanFragReq(Signal*, ScanRecord*, ScanFragRec*, bool);
+  void sendDihGetNodesLab(Signal*, ScanRecordPtr);
+  bool sendDihGetNodeReq(Signal*, ScanRecordPtr, Uint32 scanFragId);
+  void sendFragScansLab(Signal*, ScanRecordPtr);
+  bool sendScanFragReq(Signal*, ScanRecordPtr, ScanFragRecPtr);
   void sendScanTabConf(Signal* signal, ScanRecordPtr);
   void close_scan_req(Signal*, ScanRecordPtr, bool received_req);
   void close_scan_req_send_conf(Signal*, ScanRecordPtr);
@@ -1694,7 +1879,9 @@ private:
   void seizeGcp(Ptr<GcpRecord> & dst, Uint64 gci);
   void seizeTcConnect(Signal* signal);
   void seizeTcConnectFail(Signal* signal);
-  Ptr<ApiConnectRecord> sendApiCommit(Signal* signal);
+  Ptr<ApiConnectRecord> sendApiCommitAndCopy(Signal* signal);
+  void sendApiCommitSignal(Signal* signal, Ptr<ApiConnectRecord>);
+  void sendApiLateCommitSignal(Signal* signal, Ptr<ApiConnectRecord> apiCopy);
   bool sendAttrInfoTrain(Signal* signal,
                          UintR TBRef,
                          Uint32 connectPtr,
@@ -1731,9 +1918,9 @@ private:
                         const Uint32 *src,
                         Uint32 len);
   bool receivedAllTRANSID_AI(TcIndexOperation* indexOp);
-  void readIndexTable(Signal* signal, 
-		      ApiConnectRecord* regApiPtr,
-		      TcIndexOperation* indexOp,
+  void readIndexTable(Signal* signal,
+                      ApiConnectRecordPtr transPtr,
+                      TcIndexOperation* indexOp,
                       Uint32 special_op_flags);
   void executeIndexOperation(Signal* signal, 
 			     ApiConnectRecord* regApiPtr,
@@ -1755,11 +1942,13 @@ private:
                            Uint32 triggerPtrI,
                            TcConnectRecord* triggeringOp,
                            Uint32 returnCode);
-  void continueTriggeringOp(Signal* signal, TcConnectRecord* trigOp);
+  void continueTriggeringOp(Signal* signal,
+                            TcConnectRecord* trigOp,
+                            ApiConnectRecordPtr);
 
   void executeTriggers(Signal* signal, ApiConnectRecordPtr* transPtr);
   void waitToExecutePendingTrigger(Signal* signal, ApiConnectRecordPtr transPtr);
-  void executeTrigger(Signal* signal,
+  bool executeTrigger(Signal* signal,
                       TcFiredTriggerData* firedTriggerData,
                       ApiConnectRecordPtr* transPtr,
                       TcConnectRecordPtr* opPtr);
@@ -1769,11 +1958,11 @@ private:
                            ApiConnectRecordPtr* transPtr,
                            TcConnectRecordPtr* opPtr);
   Uint32 appendDataToSection(Uint32& sectionIVal,
-                             DataBuffer<11> & src,
-                             DataBuffer<11>::DataBufferIterator & iter,
+                             AttributeBuffer & src,
+                             AttributeBuffer::DataBufferIterator & iter,
                              Uint32 len);
   bool appendAttrDataToSection(Uint32& sectionIVal,
-                               DataBuffer<11>& values,
+                               AttributeBuffer& values,
                                bool withHeaders,
                                Uint32& attrId,
                                bool& hasNull);
@@ -1819,7 +2008,7 @@ private:
                              Uint32 op, Uint32 attrValuesPtrI);
 
   Uint32 fk_buildKeyInfo(Uint32& keyInfoPtrI, bool& hasNull,
-                         LocalDataBuffer<11>& values,
+                         LocalAttributeBuffer& values,
                          TcFKData* fkData,
                          bool parent);
 
@@ -1828,7 +2017,7 @@ private:
   void fk_scanFromChildTable(Signal* signal,
                              TcFiredTriggerData* firedTriggerData,
                              ApiConnectRecordPtr* transPtr,
-                             TcConnectRecordPtr* opPtr,
+                             Uint32 opPtrI,
                              TcFKData* fkData,
                              Uint32 op, Uint32 attrValuesPtrI);
   void fk_scanFromChildTable_done(Signal* signal, TcConnectRecordPtr);
@@ -1839,19 +2028,25 @@ private:
   void execKEYINFO20(Signal*);
 
   Uint32 fk_buildBounds(SegmentedSectionPtr & dst,
-                        LocalDataBuffer<11> & src,
+                        LocalAttributeBuffer & src,
                         TcFKData* fkData);
   Uint32 fk_constructAttrInfoSetNull(const TcFKData*);
   Uint32 fk_constructAttrInfoUpdateCascade(const TcFKData*,
-                                           DataBuffer<11>::Head&);
+                                           AttributeBuffer::Head&);
 
-  void releaseFiredTriggerData(DLFifoList<TcFiredTriggerData>* triggers);
-  void releaseFiredTriggerData(LocalDLFifoList<TcFiredTriggerData>* triggers);
+  bool executeFullyReplicatedTrigger(Signal* signal,
+                                     TcDefinedTriggerData* definedTriggerData,
+                                     TcFiredTriggerData* firedTriggerData,
+                                     ApiConnectRecordPtr* transPtr,
+                                     TcConnectRecordPtr* opPtr);
+
+  void releaseFiredTriggerData(TcFiredTriggerData_fifo* triggers);
+  void releaseFiredTriggerData(Local_TcFiredTriggerData_fifo* triggers);
   void abortTransFromTrigger(Signal* signal, const ApiConnectRecordPtr& transPtr, 
                              Uint32 error);
   // Generated statement blocks
   void warningHandlerLab(Signal* signal, int line);
-  void systemErrorLab(Signal* signal, int line);
+  [[noreturn]] void systemErrorLab(Signal* signal, int line);
   void sendSignalErrorRefuseLab(Signal* signal);
   void scanTabRefLab(Signal* signal, Uint32 errCode);
   void diFcountReqLab(Signal* signal, ScanRecordPtr);
@@ -1872,6 +2067,7 @@ private:
   void packLqhkeyreq040Lab(Signal* signal,
                            BlockReference TBRef);
   void returnFromQueuedDeliveryLab(Signal* signal);
+  void insert_take_over_failed_node(Signal*, Uint32 failedNodeId);
   void startTakeOverLab(Signal* signal,
                         Uint32 instanceId,
                         Uint32 failedNodeId);
@@ -1914,7 +2110,7 @@ private:
   void startphase1x010Lab(Signal* signal);
 
   void lqhKeyConf_checkTransactionState(Signal * signal,
-					Ptr<ApiConnectRecord> regApiPtr);
+                                        Ptr<ApiConnectRecord> regApiPtr);
 
   void checkDropTab(Signal* signal);
 
@@ -1922,7 +2118,7 @@ private:
 				  Uint32 scanPtrI,
 				  Uint32 failedNodeId);
   void checkScanFragList(Signal*, Uint32 failedNodeId, ScanRecord * scanP, 
-			 LocalDLList<ScanFragRec>::Head&);
+                         Local_ScanFragRec_dllist::Head&);
 
   void nodeFailCheckTransactions(Signal*,Uint32 transPtrI,Uint32 failedNodeId);
   void ndbdFailBlockCleanupCallback(Signal* signal, Uint32 failedNodeId, Uint32 ignoredRc);
@@ -1935,11 +2131,44 @@ private:
   void initData();
   void initRecords();
 
+  /**
+   * Functions used at completion of activities tracked for timing.
+   * Currently we track
+   * 1) Transactions
+   * 2) Key operations
+   * 3) Scan operations
+   * 4) Execution of SCAN_FRAGREQ's (local scans)
+   */
+   void time_track_init_histogram_limits(void);
+   Uint32 time_track_calculate_histogram_position(NDB_TICKS & start_ticks);
+
+   void time_track_complete_scan(ScanRecord * const scanPtr,
+                                 Uint32 apiNodeId);
+   void time_track_complete_scan_error(ScanRecord * const scanPtr,
+                                       Uint32 apiNodeId);
+   void time_track_complete_key_operation(TcConnectRecord * const tcConnectPtr,
+                                          Uint32 apiNodeId,
+                                          Uint32 dbNodeId);
+   void time_track_complete_index_key_operation(
+     TcConnectRecord * const tcConnectPtr,
+     Uint32 apiNodeId,
+     Uint32 dbNodeId);
+   void time_track_complete_key_operation_error(
+     TcConnectRecord * const tcConnectPtr,
+     Uint32 apiNodeId,
+     Uint32 dbNodeId);
+   void time_track_complete_scan_frag(ScanFragRec * const scanFragPtr);
+   void time_track_complete_scan_frag_error(ScanFragRec *const scanFragPtr);
+   void time_track_complete_transaction(ApiConnectRecord *const apiConnectPtr);
+   void time_track_complete_transaction_error(
+     ApiConnectRecord * const apiConnectPtr);
+   Uint32 check_own_location_domain(Uint16*, Uint32);
 protected:
   virtual bool getParam(const char* name, Uint32* count);
   
 private:
-
+  Uint32 c_time_track_histogram_boundary[TIME_TRACK_HISTOGRAM_RANGES];
+  bool c_time_track_activated;
   // Transit signals
 
 
@@ -2078,8 +2307,10 @@ private:
   Uint16 terrorCode;
 
   UintR cfirstfreeTcConnect;
-  UintR cfirstfreeApiConnectCopy;
+  UintR cfirstfreeApiConnectCopy; /* CS_RESTART */
   UintR cfirstfreeCacheRec;
+  Uint32 cfirstApiConnectPREPARE_TO_COMMIT;
+  Uint32 clastApiConnectPREPARE_TO_COMMIT;
 
   UintR cfirstgcp;
   UintR clastgcp;
@@ -2100,16 +2331,14 @@ private:
   ScanRecord *scanRecord;
   UintR cscanrecFileSize;
 
-  UnsafeArrayPool<ScanFragRec> c_scan_frag_pool;
+  ScanFragRec_pool c_scan_frag_pool;
   RSS_AP_SNAPSHOT(c_scan_frag_pool);
   ScanFragRecPtr scanFragptr;
 
   UintR cscanFragrecFileSize;
 
-  BlockReference cdictblockref;
-  BlockReference cerrorBlockref;
-  BlockReference clqhblockref;
   BlockReference cndbcntrblockref;
+  BlockInstance cspjInstanceRR;    // SPJ instance round-robin counter
 
   Uint16 csignalKey;
   Uint16 csystemnodes;
@@ -2147,21 +2376,12 @@ private:
   BlockReference tblockref;
 
   Uint8 tcurrentReplicaNo;
-  Uint8 tpad1;
-
-  UintR tapplOprec;
 
   UintR tindex;
   UintR tmaxData;
-  UintR tmp;
 
-  UintR tnodes;
   BlockReference tusersblkref;
   UintR tuserpointer;
-  UintR tloadCode;
-
-  UintR tconfig1;
-  UintR tconfig2;
 
   UintR ctransidFailHash[TRANSID_FAIL_HASH_SIZE];
   UintR ctcConnectFailHash[TC_FAIL_HASH_SIZE];
@@ -2197,10 +2417,12 @@ public:
 
 private:
   typedef Ptr<CommitAckMarker> CommitAckMarkerPtr;
-  typedef DLHashTable<CommitAckMarker>::Iterator CommitAckMarkerIterator;
+  typedef ArrayPool<CommitAckMarker> CommitAckMarker_pool;
+  typedef DLHashTable<CommitAckMarker_pool> CommitAckMarker_hash;
+  typedef CommitAckMarker_hash::Iterator CommitAckMarkerIterator;
   
-  ArrayPool<CommitAckMarker>   m_commitAckMarkerPool;
-  DLHashTable<CommitAckMarker> m_commitAckMarkerHash;
+  CommitAckMarker_pool m_commitAckMarkerPool;
+  CommitAckMarker_hash m_commitAckMarkerHash;
   RSS_AP_SNAPSHOT(m_commitAckMarkerPool);
   
   void execTC_COMMIT_ACK(Signal* signal);
@@ -2226,7 +2448,7 @@ private:
   
   void checkAbortAllTimeout(Signal* signal, Uint32 sleepTime);
   struct AbortAllRecord {
-    AbortAllRecord(){ clientRef = 0; }
+    AbortAllRecord() : clientRef(0), oldTimeOutValue(0) {}
     Uint32 clientData;
     BlockReference clientRef;
     
@@ -2347,6 +2569,8 @@ private:
 
   Uint32 c_sttor_ref;
 
+  Uint32 m_load_balancer_location;
+
 #ifdef ERROR_INSERT
   // Used with ERROR_INSERT 8078 + 8079 to check API_FAILREQ handling
   Uint32 c_lastFailedApi;
