@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -24,6 +24,7 @@
 
 #include "Trix.hpp"
 
+#include <cstring>
 #include <string.h>
 #include <kernel_types.h>
 #include <NdbOut.hpp>
@@ -53,7 +54,7 @@
 
 #define CONSTRAINT_VIOLATION 893
 #define TUPLE_NOT_FOUND 626
-#define FK_NO_PARENT_ROW_EXISTS 255
+#define FK_NO_PARENT_ROW_EXISTS 21033
 
 static
 bool
@@ -153,12 +154,24 @@ Trix::execREAD_CONFIG_REQ(Signal* signal)
     m_ctx.m_config.getOwnConfigIterator();
   ndbrequire(p != 0);
 
+  c_maxUIBuildBatchSize = 64;
+  ndb_mgm_get_int_parameter(p, CFG_DB_UI_BUILD_MAX_BATCHSIZE,
+                            &c_maxUIBuildBatchSize);
+
+  c_maxFKBuildBatchSize = 64;
+  ndb_mgm_get_int_parameter(p, CFG_DB_FK_BUILD_MAX_BATCHSIZE,
+                            &c_maxFKBuildBatchSize);
+
+  c_maxReorgBuildBatchSize = 64;
+  ndb_mgm_get_int_parameter(p, CFG_DB_REORG_BUILD_MAX_BATCHSIZE,
+                            &c_maxReorgBuildBatchSize);
+
   // Allocate pool sizes
   c_theAttrOrderBufferPool.setSize(100);
   c_theSubscriptionRecPool.setSize(100);
   c_statOpPool.setSize(5);
 
-  DLList<SubscriptionRecord> subscriptions(c_theSubscriptionRecPool);
+  SubscriptionRecord_list subscriptions(c_theSubscriptionRecPool);
   SubscriptionRecPtr subptr;
   while (subscriptions.seizeFirst(subptr) == true) {
     new (subptr.p) SubscriptionRecord(c_theAttrOrderBufferPool);
@@ -339,11 +352,11 @@ Trix::execDUMP_STATE_ORD(Signal* signal)
     // index2 -T; index2 -I -n10000; index2 -c
     // all dump 300 0 0 0 0 0 4 2
     // select_count INDEX0000
+    std::memmove(signal->theData,
+                 signal->theData + 1,
+                 BuildIndxImplReq::SignalLength * sizeof(signal->theData[0]));
     BuildIndxImplReq * buildIndxReq = (BuildIndxImplReq *)signal->getDataPtrSend();
     
-    MEMCOPY_NO_WORDS(buildIndxReq, 
-		     signal->theData + 1, 
-		     BuildIndxImplReq::SignalLength);
     buildIndxReq->senderRef = reference(); // return to me
     buildIndxReq->parallelism = 10;
     Uint32 indexColumns[1] = {1};
@@ -364,11 +377,11 @@ Trix::execDUMP_STATE_ORD(Signal* signal)
     // index2 -T; index2 -I -n10000; index2 -c -p
     // all dump 301 0 0 0 0 0 4 2
     // select_count INDEX0000
+    std::memmove(signal->theData,
+                 signal->theData + 1,
+                 BuildIndxImplReq::SignalLength * sizeof(signal->theData[0]));
     BuildIndxImplReq * buildIndxReq = (BuildIndxImplReq *)signal->getDataPtrSend();
     
-    MEMCOPY_NO_WORDS(buildIndxReq, 
-		     signal->theData + 1, 
-		     BuildIndxImplReq::SignalLength);
     buildIndxReq->senderRef = reference(); // return to me
     buildIndxReq->parallelism = 10;
     Uint32 indexColumns[2] = {0, 1};
@@ -389,11 +402,11 @@ Trix::execDUMP_STATE_ORD(Signal* signal)
     // index -T; index -I -n1000; index -c -p
     // all dump 302 0 0 0 0 0 4 2
     // select_count PNUMINDEX0000
+    std::memmove(signal->theData,
+                 signal->theData + 1,
+                 BuildIndxImplReq::SignalLength * sizeof(signal->theData[0]));
     BuildIndxImplReq * buildIndxReq = (BuildIndxImplReq *)signal->getDataPtrSend();
     
-    MEMCOPY_NO_WORDS(buildIndxReq, 
-		     signal->theData + 1, 
-		     BuildIndxImplReq::SignalLength);
     buildIndxReq->senderRef = reference(); // return to me
     buildIndxReq->parallelism = 10;
     Uint32 indexColumns[3] = {0, 3, 5};
@@ -414,11 +427,11 @@ Trix::execDUMP_STATE_ORD(Signal* signal)
     // index -T -2; index -I -2 -n1000; index -c -p
     // all dump 303 0 0 0 0 0 4 2
     // select_count PNUMINDEX0000
+    std::memmove(signal->theData,
+                 signal->theData + 1,
+                 BuildIndxImplReq::SignalLength * sizeof(signal->theData[0]));
     BuildIndxImplReq * buildIndxReq = (BuildIndxImplReq *)signal->getDataPtrSend();
     
-    MEMCOPY_NO_WORDS(buildIndxReq, 
-		     signal->theData + 1, 
-		     BuildIndxImplReq::SignalLength);
     buildIndxReq->senderRef = reference(); // return to me
     buildIndxReq->parallelism = 10;
     Uint32 indexColumns[3] = {0, 3, 5};
@@ -439,11 +452,11 @@ Trix::execDUMP_STATE_ORD(Signal* signal)
     // index -T -L; index -I -L -n1000; index -c -p
     // all dump 304 0 0 0 0 0 4 2
     // select_count PNUMINDEX0000
+    std::memmove(signal->theData,
+                 signal->theData + 1,
+                 BuildIndxImplReq::SignalLength * sizeof(signal->theData[0]));
     BuildIndxImplReq * buildIndxReq = (BuildIndxImplReq *)signal->getDataPtrSend();
     
-    MEMCOPY_NO_WORDS(buildIndxReq, 
-		     signal->theData + 1, 
-		     BuildIndxImplReq::SignalLength);
     buildIndxReq->senderRef = reference(); // return to me
     buildIndxReq->parallelism = 10;
     Uint32 indexColumns[3] = {0, 3, 5};
@@ -464,11 +477,11 @@ Trix::execDUMP_STATE_ORD(Signal* signal)
     // index -T -2 -L; index -I -2 -L -n1000; index -c -p
     // all dump 305 0 0 0 0 0 4 2
     // select_count PNUMINDEX0000
+    std::memmove(signal->theData,
+                 signal->theData + 1,
+                 BuildIndxImplReq::SignalLength * sizeof(signal->theData[0]));
     BuildIndxImplReq * buildIndxReq = (BuildIndxImplReq *)signal->getDataPtrSend();
     
-    MEMCOPY_NO_WORDS(buildIndxReq, 
-		     signal->theData + 1, 
-		     BuildIndxImplReq::SignalLength);
     buildIndxReq->senderRef = reference(); // return to me
     buildIndxReq->parallelism = 10;
     Uint32 indexColumns[3] = {0, 3, 5};
@@ -624,7 +637,7 @@ void Trix:: execBUILD_INDX_IMPL_REQ(Signal* signal)
   subRec->indexType = buildIndxReq->indexType;
   subRec->sourceTableId = buildIndxReq->tableId;
   subRec->targetTableId = buildIndxReq->indexId;
-  subRec->parallelism = buildIndxReq->parallelism;
+  subRec->parallelism = c_maxUIBuildBatchSize;
   subRec->expectedConf = 0;
   subRec->subscriptionCreated = false;
   subRec->pendingSubSyncContinueConf = false;
@@ -719,6 +732,19 @@ void Trix::execUTIL_PREPARE_REF(Signal* signal)
   }
   subRecPtr.p = subRec;
   subRec->errorCode = (BuildIndxRef::ErrorCode)utilPrepareRef->errorCode;
+  switch (utilPrepareRef->errorCode) {
+  case UtilPrepareRef::PREPARE_SEIZE_ERROR:
+  case UtilPrepareRef::PREPARE_PAGES_SEIZE_ERROR:
+  case UtilPrepareRef::PREPARED_OPERATION_SEIZE_ERROR:
+  case UtilPrepareRef::DICT_TAB_INFO_ERROR:
+    subRec->errorCode = BuildIndxRef::UtilBusy;
+    break;
+  case UtilPrepareRef::MISSING_PROPERTIES_SECTION:
+    subRec->errorCode = BuildIndxRef::BadRequestType;
+    break;
+  default:
+    ndbabort();
+  }
 
   UtilReleaseConf* conf = (UtilReleaseConf*)signal->getDataPtrSend();
   conf->senderData = subRecPtr.i;
@@ -956,8 +982,7 @@ void Trix::execSUB_TABLE_DATA(Signal* signal)
     executeBuildFKTransaction(signal, subRecPtr);
     break;
   case STAT_UTIL:
-    ndbrequire(false);
-    break;
+    ndbabort();
   case STAT_CLEAN:
     {
       StatOp& stat = statOpGetPtr(subRecPtr.p->m_statPtrI);
@@ -996,6 +1021,8 @@ void Trix::setupSubscription(Signal* signal, SubscriptionRecPtr subRecPtr)
 		     subRecPtr.i, subCreateReq->subscriptionId,
 		     subCreateReq->subscriptionKey));
 
+  D("SUB_CREATE_REQ tableId: " << subRec->sourceTableId);
+
   sendSignal(SUMA_REF, GSN_SUB_CREATE_REQ, 
 	     signal, SubCreateReq::SignalLength, JBB);
 
@@ -1025,7 +1052,7 @@ void Trix::startTableScan(Signal* signal, SubscriptionRecPtr subRecPtr)
   }
 
   // Merge index and key column segments
-  struct LinearSectionPtr orderPtr[3];
+  LinearSectionPtr orderPtr[3];
   Uint32 noOfSections;
   orderPtr[0].p = attributeList;
   orderPtr[0].sz = cnt;
@@ -1040,6 +1067,7 @@ void Trix::startTableScan(Signal* signal, SubscriptionRecPtr subRecPtr)
   subSyncReq->requestInfo = 0;
   subSyncReq->fragCount = subRec->fragCount;
   subSyncReq->fragId = subRec->fragId;
+  subSyncReq->batchSize = subRec->parallelism;
 
   if (subRec->m_flags & SubscriptionRecord::RF_NO_DISK)
   {
@@ -1062,13 +1090,13 @@ void Trix::startTableScan(Signal* signal, SubscriptionRecPtr subRecPtr)
   {
     jam();
     subSyncReq->requestInfo |= SubSyncReq::LM_Exclusive;
-    subSyncReq->requestInfo |= SubSyncReq::Reorg;
+    subSyncReq->requestInfo |= SubSyncReq::ReorgDelete;
   }
   else if (subRec->requestType == STAT_CLEAN)
   {
     jam();
     StatOp& stat = statOpGetPtr(subRecPtr.p->m_statPtrI);
-    StatOp::Clean clean = stat.m_clean;
+    StatOp::Clean& clean = stat.m_clean;
     orderPtr[1].p = clean.m_bound;
     orderPtr[1].sz = clean.m_boundSize;
     noOfSections = 2;
@@ -1091,6 +1119,10 @@ void Trix::startTableScan(Signal* signal, SubscriptionRecPtr subRecPtr)
 		     subRecPtr.i, subSyncReq->subscriptionId,
 		     subSyncReq->subscriptionKey));
 
+  D("SUB_SYNC_REQ fragId: " << subRec->fragId <<
+    " fragCount: " << subRec->fragCount <<
+    " requestInfo: " << hex << subSyncReq->requestInfo);
+
   sendSignal(SUMA_REF, GSN_SUB_SYNC_REQ,
 	     signal, SubSyncReq::SignalLength, JBB, orderPtr, noOfSections);
 }
@@ -1331,7 +1363,7 @@ void
 Trix::execSUB_REMOVE_REF(Signal* signal){
   jamEntry();
   //@todo
-  ndbrequire(false);
+  ndbabort();
 }
 
 void
@@ -1365,7 +1397,7 @@ Trix::execSUB_REMOVE_CONF(Signal* signal){
 void
 Trix::execUTIL_RELEASE_REF(Signal* signal){
   jamEntry();
-  ndbrequire(false);
+  ndbabort();
 }
 
 void
@@ -1549,7 +1581,7 @@ Trix::execCOPY_DATA_IMPL_REQ(Signal* signal)
   subRec->indexType = RNIL;
   subRec->sourceTableId = req->srcTableId;
   subRec->targetTableId = req->dstTableId;
-  subRec->parallelism = 16;
+  subRec->parallelism = c_maxReorgBuildBatchSize;
   subRec->expectedConf = 0;
   subRec->subscriptionCreated = false;
   subRec->pendingSubSyncContinueConf = false;
@@ -1569,7 +1601,7 @@ Trix::execCOPY_DATA_IMPL_REQ(Signal* signal)
     break;
   default:
     jamLine(req->requestType);
-    ndbrequire(false);
+    ndbabort();
   }
 
   if (req->requestInfo & CopyDataReq::TupOrder)
@@ -1596,6 +1628,12 @@ Trix::execCOPY_DATA_IMPL_REQ(Signal* signal)
     subRec->noOfKeyColumns = ptr.sz;
   }
 
+  D("COPY_DATA_IMPL_REQ srctableId: " << subRec->sourceTableId <<
+    " targetTableId: " << subRec->targetTableId <<
+    " fragCount: " << subRec->fragCount <<
+    " requestType: " << subRec->requestType <<
+    " flags: " << hex << subRec->m_flags);
+
   releaseSections(handle);
   {
     UtilPrepareReq * utilPrepareReq =
@@ -1618,7 +1656,10 @@ Trix::execCOPY_DATA_IMPL_REQ(Signal* signal)
     {
       w.add(UtilPrepareReq::OperationType, UtilPrepareReq::Delete);
     }
-    w.add(UtilPrepareReq::ScanTakeOverInd, 1);
+    if (!(req->requestInfo & CopyDataReq::NoScanTakeOver))
+    {
+      w.add(UtilPrepareReq::ScanTakeOverInd, 1);
+    }
     w.add(UtilPrepareReq::ReorgInd, 1);
     w.add(UtilPrepareReq::TableId, subRec->targetTableId);
 
@@ -1678,7 +1719,7 @@ Trix::execBUILD_FK_IMPL_REQ(Signal* signal)
   subRec->indexType = RNIL;
   subRec->sourceTableId = req->childTableId;
   subRec->targetTableId = req->parentTableId;
-  subRec->parallelism = 16;
+  subRec->parallelism = c_maxFKBuildBatchSize;
   subRec->expectedConf = 0;
   subRec->subscriptionCreated = false;
   subRec->pendingSubSyncContinueConf = false;
@@ -1914,8 +1955,7 @@ Trix::execINDEX_STAT_IMPL_REQ(Signal* signal)
     stat.m_requestName = "drop head";
     break;
   default:
-    ndbrequire(false);
-    break;
+    ndbabort();
   }
 
   SubscriptionRecord* subRec = c_theSubscriptions.getPtr(stat.m_subRecPtrI);
@@ -2341,8 +2381,7 @@ Trix::statUtilPrepareRef(Signal* signal, Uint32 statPtrI)
     break;
   case UtilPrepareRef::MISSING_PROPERTIES_SECTION:
   default:
-    ndbrequire(false);
-    break;
+    ndbabort();
   }
   statOpError(signal, stat, errorCode, __LINE__);
 }
@@ -2430,8 +2469,7 @@ Trix::statUtilExecuteRef(Signal* signal, Uint32 statPtrI)
     errorCode = IndexStatRef::BusyUtilExecute;
     break;
   default:
-    ndbrequire(false);
-    break;
+    ndbabort();
   }
 
   if (errorCode != 0)
@@ -2476,8 +2514,10 @@ Trix::statReadHeadDone(Signal* signal, StatOp& stat)
   switch (stat.m_requestType) {
   case IndexStatReq::RT_CLEAN_NEW:
     jam();
+    // Fall through
   case IndexStatReq::RT_CLEAN_OLD:
     jam();
+    // Fall through
   case IndexStatReq::RT_CLEAN_ALL:
     jam();
     statCleanBegin(signal, stat);
@@ -2494,8 +2534,7 @@ Trix::statReadHeadDone(Signal* signal, StatOp& stat)
     break;
 
   default:
-    ndbrequire(false);
-    break;
+    ndbabort();
   }
 }
 
@@ -2510,8 +2549,7 @@ Trix::statInsertHeadDone(Signal* signal, StatOp& stat)
     statScanEnd(signal, stat);
     break;
   default:
-    ndbrequire(false);
-    break;
+    ndbabort();
   }
 }
 
@@ -2526,8 +2564,7 @@ Trix::statUpdateHeadDone(Signal* signal, StatOp& stat)
     statScanEnd(signal, stat);
     break;
   default:
-    ndbrequire(false);
-    break;
+    ndbabort();
   }
 }
 
@@ -2542,8 +2579,7 @@ Trix::statDeleteHeadDone(Signal* signal, StatOp& stat)
     statDropEnd(signal, stat);
     break;
   default:
-    ndbrequire(false);
-    break;
+    ndbabort();
   }
 }
 
@@ -2615,7 +2651,7 @@ Trix::statCleanPrepare(Signal* signal, StatOp& stat)
   subRec->targetTableId = RNIL;
   subRec->noOfIndexColumns = ao_size;
   subRec->noOfKeyColumns = 0;
-  subRec->parallelism = 16;
+  subRec->parallelism = 16;  // remains hardcoded for now
   subRec->fragCount = 0;
   subRec->fragId = ZNIL;
   subRec->syncPtr = RNIL;
@@ -2637,30 +2673,31 @@ Trix::statCleanPrepare(Signal* signal, StatOp& stat)
   clean.m_bound[3] = TuxBoundInfo::BoundEQ;
   clean.m_bound[4] = AttributeHeader(1, 4).m_value;
   clean.m_bound[5] = data.m_indexVersion;
+  Uint32 boundCount;
   switch (stat.m_requestType) {
   case IndexStatReq::RT_CLEAN_NEW:
     D("statCleanPrepare delete sample versions > " << data.m_sampleVersion);
     clean.m_bound[6] = TuxBoundInfo::BoundLT;
     clean.m_bound[7] = AttributeHeader(2, 4).m_value;
     clean.m_bound[8] = data.m_sampleVersion;
-    clean.m_boundCount = 3;
+    boundCount = 3;
     break;
   case IndexStatReq::RT_CLEAN_OLD:
     D("statCleanPrepare delete sample versions < " << data.m_sampleVersion);
     clean.m_bound[6] = TuxBoundInfo::BoundGT;
     clean.m_bound[7] = AttributeHeader(2, 4).m_value;
     clean.m_bound[8] = data.m_sampleVersion;
-    clean.m_boundCount = 3;
+    boundCount = 3;
     break;
   case IndexStatReq::RT_CLEAN_ALL:
     D("statCleanPrepare delete all sample versions");
-    clean.m_boundCount = 2;
+    boundCount = 2;
     break;
   default:
-    ndbrequire(false);
-    break;
+    boundCount = 0; /* Silence compiler warning */
+    ndbabort();
   }
-  clean.m_boundSize = 3 * clean.m_boundCount;
+  clean.m_boundSize = 3 * boundCount;
 
   // TRIX traps the CONF
   send.m_sysTable = &g_statMetaSample;
@@ -2822,7 +2859,7 @@ Trix::statScanPrepare(Signal* signal, StatOp& stat)
   subRec->targetTableId = RNIL;
   subRec->noOfIndexColumns = ao_size;
   subRec->noOfKeyColumns = 0;
-  subRec->parallelism = 16;
+  subRec->parallelism = 16;   // remains hardcoded for now
   subRec->fragCount = 0; // XXX Suma currently checks all frags
   subRec->fragId = req->fragId;
   subRec->syncPtr = RNIL;
@@ -2962,7 +2999,7 @@ Trix::statScanEnd(Signal* signal, StatOp& stat)
    * we prefer DbtuxProxy to avoid introducing MT-LQH into TRIX.
    */
 
-#if trix_index_stat_rep_to_tux_instance
+#ifdef trix_index_stat_rep_to_tux_instance
   Uint32 instanceKey = getInstanceKey(req->indexId, req->fragId);
   BlockReference tuxRef = numberToRef(DBTUX, instanceKey, getOwnNodeId());
 #else
@@ -3049,8 +3086,7 @@ Trix::statSendPrepare(Signal* signal, StatOp& stat)
         w.add(UtilPrepareReq::AttributeId, i);
       break;
     default:
-      ndbrequire(false);
-      break;
+      ndbabort();
     }
   }
 
@@ -3100,8 +3136,7 @@ Trix::statSendExecute(Signal* signal, StatOp& stat)
         statDataOut(stat, i);
       break;
     default:
-      ndbrequire(false);
-      break;
+      ndbabort();
     }
   }
 
@@ -3181,8 +3216,7 @@ Trix::statDataPtr(StatOp& stat, Uint32 i, Uint32*& dptr, Uint32& bytes)
       bytes = 4;
       break;
     default:
-      ndbrequire(false);
-      break;
+      ndbabort();
     }
     return;
   }
@@ -3219,13 +3253,12 @@ Trix::statDataPtr(StatOp& stat, Uint32 i, Uint32*& dptr, Uint32& bytes)
       }
       break;
     default:
-      ndbrequire(false);
-      break;
+      ndbabort();
     }
     return;
   }
 
-  ndbrequire(false);
+  ndbabort();
 }
 
 void
@@ -3430,5 +3463,3 @@ operator<<(NdbOut& out, const Trix::StatOp& stat)
 
 
 BLOCK_FUNCTIONS(Trix)
-
-template void append(DataBuffer<15>&,SegmentedSectionPtr,SectionSegmentPool&);
