@@ -1,4 +1,4 @@
-/* Copyright (c) 2008, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -18,7 +18,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 
 #include "ConfigManager.hpp"
@@ -627,7 +627,8 @@ check_no_dynamic_ports_in_config(const Config* config)
   ConfigIter iter(config, CFG_SECTION_CONNECTION);
 
   for(;iter.valid();iter.next()) {
-    Uint32 n1, n2;
+    Uint32 n1 = 0;
+    Uint32 n2 = 0;
     require(iter.get(CFG_CONNECTION_NODE_1, &n1) == 0 &&
             iter.get(CFG_CONNECTION_NODE_2, &n2) == 0);
 
