@@ -1,4 +1,4 @@
-/* Copyright (c) 2008, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2018, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -17,8 +17,8 @@
   GNU General Public License, version 2.0, for more details.
 
   You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software Foundation,
-  51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA */
+  along with this program; if not, write to the Free Software
+  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #ifndef PFS_SERVER_H
 #define PFS_SERVER_H
@@ -28,45 +28,70 @@
   Private interface for the server (declarations).
 */
 
+#include <sys/types.h>
+
+#include "my_psi_config.h"
+#include "mysql/psi/psi_cond.h"
+#include "mysql/psi/psi_data_lock.h"
+#include "mysql/psi/psi_error.h"
+#include "mysql/psi/psi_file.h"
+#include "mysql/psi/psi_idle.h"
+#include "mysql/psi/psi_mdl.h"
+#include "mysql/psi/psi_memory.h"
+#include "mysql/psi/psi_mutex.h"
+#include "mysql/psi/psi_rwlock.h"
+#include "mysql/psi/psi_socket.h"
+#include "mysql/psi/psi_stage.h"
+#include "mysql/psi/psi_statement.h"
+#include "mysql/psi/psi_system.h"
+#include "mysql/psi/psi_table.h"
+#include "mysql/psi/psi_thread.h"
+#include "mysql/psi/psi_transaction.h"
+
+#ifdef HAVE_PSI_INTERFACE
+
 #define PFS_AUTOSCALE_VALUE (-1)
 #define PFS_AUTOSIZE_VALUE (-1)
 
 #ifndef PFS_MAX_MUTEX_CLASS
-  #define PFS_MAX_MUTEX_CLASS 250
+#define PFS_MAX_MUTEX_CLASS 340
 #endif
 #ifndef PFS_MAX_RWLOCK_CLASS
-  #define PFS_MAX_RWLOCK_CLASS 50
+#define PFS_MAX_RWLOCK_CLASS 70
 #endif
 #ifndef PFS_MAX_COND_CLASS
-  #define PFS_MAX_COND_CLASS 80
+#define PFS_MAX_COND_CLASS 100
 #endif
 #ifndef PFS_MAX_THREAD_CLASS
-  #define PFS_MAX_THREAD_CLASS 50
+#define PFS_MAX_THREAD_CLASS 100
 #endif
 #ifndef PFS_MAX_FILE_CLASS
-  #define PFS_MAX_FILE_CLASS 80
+#define PFS_MAX_FILE_CLASS 80
 #endif
 #ifndef PFS_MAX_FILE_HANDLE
-  #define PFS_MAX_FILE_HANDLE 32768
+#define PFS_MAX_FILE_HANDLE 32768
 #endif
 #ifndef PFS_MAX_SOCKET_CLASS
-  #define PFS_MAX_SOCKET_CLASS 10
+#define PFS_MAX_SOCKET_CLASS 10
 #endif
 #ifndef PFS_MAX_STAGE_CLASS
-  #define PFS_MAX_STAGE_CLASS 150
+#define PFS_MAX_STAGE_CLASS 175
 #endif
 #ifndef PFS_STATEMENTS_STACK_SIZE
-  #define PFS_STATEMENTS_STACK_SIZE 10
+#define PFS_STATEMENTS_STACK_SIZE 10
 #endif
 #ifndef PFS_MAX_MEMORY_CLASS
-  #define PFS_MAX_MEMORY_CLASS 400
+#define PFS_MAX_MEMORY_CLASS 530
+#endif
+
+#ifndef PFS_MAX_SERVER_ERRORS
+#define PFS_MAX_SERVER_ERRORS ((total_error_count - obsolete_error_count) + 1)
 #endif
 
 /** Sizing hints, from the server configuration. */
-struct PFS_sizing_hints
-{
+struct PFS_sizing_hints {
   /** Value of @c Sys_table_def_size */
-  long m_table_definition_cache;
+  ulong m_table_definition_cache;
   /** Value of @c Sys_table_cache_size */
   long m_table_open_cache;
   /** Value of @c Sys_max_connections */
@@ -78,8 +103,7 @@ struct PFS_sizing_hints
 };
 
 /** Performance schema global sizing parameters. */
-struct PFS_global_param
-{
+struct PFS_global_param {
   /** True if the performance schema is enabled. */
   bool m_enabled;
   /** Default values for SETUP_CONSUMERS. */
@@ -178,7 +202,7 @@ struct PFS_global_param
   */
   long m_file_handle_sizing;
   /**
-    Maxium number of instrumented socket instances
+    Maximum number of instrumented socket instances
     @sa socket_lost
   */
   long m_socket_sizing;
@@ -246,6 +270,12 @@ struct PFS_global_param
   long m_max_digest_length;
   ulong m_max_sql_text_length;
 
+  /** Maximum age in seconds for a query sample. */
+  ulong m_max_digest_sample_age;
+
+  /** Maximum number of error instrumented */
+  ulong m_error_sizing;
+
   /** Sizing hints, for auto tuning. */
   PFS_sizing_hints m_hints;
 };
@@ -266,11 +296,44 @@ void pre_initialize_performance_schema();
 
 /**
   Initialize the performance schema.
+  The performance schema implement several instrumentation services.
+  Each instrumentation service is versioned, and accessible through
+  a bootstrap structure, returned as output parameter.
   @param param Size parameters to use.
-  @return A bootstrap handle, or NULL.
+  @param [out] thread_bootstrap Thread instrumentation service bootstrap
+  @param [out] mutex_bootstrap Mutex instrumentation service bootstrap
+  @param [out] rwlock_bootstrap Rwlock instrumentation service bootstrap
+  @param [out] cond_bootstrap Condition instrumentation service bootstrap
+  @param [out] file_bootstrap File instrumentation service bootstrap
+  @param [out] socket_bootstrap Socket instrumentation service bootstrap
+  @param [out] table_bootstrap Table instrumentation service bootstrap
+  @param [out] mdl_bootstrap Metadata Lock instrumentation service bootstrap
+  @param [out] idle_bootstrap Idle instrumentation service bootstrap
+  @param [out] stage_bootstrap Stage instrumentation service bootstrap
+  @param [out] statement_bootstrap Statement instrumentation service bootstrap
+  @param [out] transaction_bootstrap Transaction instrumentation service
+  bootstrap
+  @param [out] memory_bootstrap Memory instrumentation service bootstrap
+  @param [out] error_bootstrap Error instrumentation service bootstrap
+  @param [out] data_lock_bootstrap Data Lock instrumentation service bootstrap
+  @param [out] system_bootstrap System instrumentation service bootstrap
+  @returns
+    @retval 0 success
 */
-struct PSI_bootstrap*
-initialize_performance_schema(PFS_global_param *param);
+int initialize_performance_schema(
+    PFS_global_param *param, PSI_thread_bootstrap **thread_bootstrap,
+    PSI_mutex_bootstrap **mutex_bootstrap,
+    PSI_rwlock_bootstrap **rwlock_bootstrap,
+    PSI_cond_bootstrap **cond_bootstrap, PSI_file_bootstrap **file_bootstrap,
+    PSI_socket_bootstrap **socket_bootstrap,
+    PSI_table_bootstrap **table_bootstrap, PSI_mdl_bootstrap **mdl_bootstrap,
+    PSI_idle_bootstrap **idle_bootstrap, PSI_stage_bootstrap **stage_bootstrap,
+    PSI_statement_bootstrap **statement_bootstrap,
+    PSI_transaction_bootstrap **transaction_bootstrap,
+    PSI_memory_bootstrap **memory_bootstrap,
+    PSI_error_bootstrap **error_bootstrap,
+    PSI_data_lock_bootstrap **data_lock_bootstrap,
+    PSI_system_bootstrap **system_bootstrap);
 
 void pfs_automated_sizing(PFS_global_param *param);
 
@@ -285,8 +348,6 @@ void pfs_automated_sizing(PFS_global_param *param);
 */
 void initialize_performance_schema_acl(bool bootstrap);
 
-void check_performance_schema();
-
 /**
   Reset the aggregated status counter stats.
 */
@@ -301,11 +362,25 @@ void init_pfs_instrument_array();
 /**
   Process one PFS_INSTRUMENT configuration string.
 */
-int add_pfs_instr_to_array(const char* name, const char* value);
+int add_pfs_instr_to_array(const char *name, const char *value);
+
+/**
+  Register/unregister notification service.
+*/
+int register_pfs_notification_service();
+int unregister_pfs_notification_service();
+
+/**
+  Register/unregister resource group service.
+*/
+int register_pfs_resource_group_service();
+int unregister_pfs_resource_group_service();
 
 /**
   Shutdown the performance schema.
 */
 void shutdown_performance_schema();
 
-#endif
+#endif /* HAVE_PSI_INTERFACE */
+
+#endif /* PFS_SERVER_H */
