@@ -22,9 +22,8 @@
 #include <string>
 
 /* MySQL header files */
-#include "handler.h"
-#include "my_global.h"
 #include "my_io_perf.h"
+#include "sql/handler.h"
 
 namespace myrocks {
 
@@ -107,7 +106,7 @@ class Rdb_perf_counters {
   Rdb_perf_counters(const Rdb_perf_counters &) = delete;
   Rdb_perf_counters &operator=(const Rdb_perf_counters &) = delete;
 
-public:
+ public:
   Rdb_perf_counters() = default;
   uint64_t m_value[PC_MAX_IDX];
 
@@ -125,7 +124,7 @@ class Rdb_io_perf {
   my_io_perf_atomic_t *m_shared_io_perf_read = nullptr;
   ha_statistics *m_stats = nullptr;
 
-public:
+ public:
   Rdb_io_perf(const Rdb_io_perf &) = delete;
   Rdb_io_perf &operator=(const Rdb_io_perf &) = delete;
 
@@ -149,4 +148,4 @@ public:
         m_stats(nullptr) {}
 };
 
-} // namespace myrocks
+}  // namespace myrocks
