@@ -14,7 +14,7 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #ifdef USE_PRAGMA_IMPLEMENTATION
-#pragma implementation // gcc: Class implementation
+#pragma implementation  // gcc: Class implementation
 #endif
 
 #define MYSQL_SERVER 1
@@ -28,8 +28,8 @@ namespace myrocks {
   The following is needed as an argument for mysql_stage_register,
   irrespectively of whether we're compiling with P_S or not.
 */
-my_core::PSI_stage_info stage_waiting_on_row_lock = {0, "Waiting for row lock",
-                                                     0};
+my_core::PSI_stage_info stage_waiting_on_row_lock(0, "Waiting for row lock", 0,
+                                                  nullptr);
 
 #ifdef HAVE_PSI_INTERFACE
 my_core::PSI_stage_info *all_rocksdb_stages[] = {&stage_waiting_on_row_lock};
@@ -38,9 +38,9 @@ my_core::PSI_thread_key rdb_background_psi_thread_key,
     rdb_drop_idx_psi_thread_key, rdb_mc_psi_thread_key;
 
 my_core::PSI_thread_info all_rocksdb_threads[] = {
-    {&rdb_background_psi_thread_key, "background", PSI_FLAG_GLOBAL},
-    {&rdb_drop_idx_psi_thread_key, "drop index", PSI_FLAG_GLOBAL},
-    {&rdb_mc_psi_thread_key, "manual compaction", PSI_FLAG_GLOBAL},
+    {&rdb_background_psi_thread_key, "background", PSI_FLAG_SINGLETON},
+    {&rdb_drop_idx_psi_thread_key, "drop index", PSI_FLAG_SINGLETON},
+    {&rdb_mc_psi_thread_key, "manual compaction", PSI_FLAG_SINGLETON},
 };
 
 my_core::PSI_mutex_key rdb_psi_open_tbls_mutex_key, rdb_signal_bg_psi_mutex_key,
@@ -50,19 +50,22 @@ my_core::PSI_mutex_key rdb_psi_open_tbls_mutex_key, rdb_signal_bg_psi_mutex_key,
     rdb_sst_commit_key, rdb_block_cache_resize_mutex_key;
 
 my_core::PSI_mutex_info all_rocksdb_mutexes[] = {
-    {&rdb_psi_open_tbls_mutex_key, "open tables", PSI_FLAG_GLOBAL},
-    {&rdb_signal_bg_psi_mutex_key, "stop background", PSI_FLAG_GLOBAL},
-    {&rdb_signal_drop_idx_psi_mutex_key, "signal drop index", PSI_FLAG_GLOBAL},
-    {&rdb_signal_mc_psi_mutex_key, "signal manual compaction", PSI_FLAG_GLOBAL},
-    {&rdb_collation_data_mutex_key, "collation data init", PSI_FLAG_GLOBAL},
+    {&rdb_psi_open_tbls_mutex_key, "open tables", PSI_FLAG_SINGLETON},
+    {&rdb_signal_bg_psi_mutex_key, "stop background", PSI_FLAG_SINGLETON},
+    {&rdb_signal_drop_idx_psi_mutex_key, "signal drop index",
+     PSI_FLAG_SINGLETON},
+    {&rdb_collation_data_mutex_key, "collation data init", PSI_FLAG_SINGLETON},
+    {&rdb_signal_mc_psi_mutex_key, "signal manual compaction",
+     PSI_FLAG_SINGLETON},
+    {&rdb_collation_data_mutex_key, "collation data init", PSI_FLAG_SINGLETON},
     {&rdb_mem_cmp_space_mutex_key, "collation space char data init",
-     PSI_FLAG_GLOBAL},
-    {&key_mutex_tx_list, "tx_list", PSI_FLAG_GLOBAL},
-    {&rdb_sysvars_psi_mutex_key, "setting sysvar", PSI_FLAG_GLOBAL},
-    {&rdb_cfm_mutex_key, "column family manager", PSI_FLAG_GLOBAL},
-    {&rdb_sst_commit_key, "sst commit", PSI_FLAG_GLOBAL},
+     PSI_FLAG_SINGLETON},
+    {&key_mutex_tx_list, "tx_list", PSI_FLAG_SINGLETON},
+    {&rdb_sysvars_psi_mutex_key, "setting sysvar", PSI_FLAG_SINGLETON},
+    {&rdb_cfm_mutex_key, "column family manager", PSI_FLAG_SINGLETON},
+    {&rdb_sst_commit_key, "sst commit", PSI_FLAG_SINGLETON},
     {&rdb_block_cache_resize_mutex_key, "resizing block cache",
-     PSI_FLAG_GLOBAL},
+     PSI_FLAG_SINGLETON},
 };
 
 my_core::PSI_rwlock_key key_rwlock_collation_exception_list,
@@ -70,21 +73,22 @@ my_core::PSI_rwlock_key key_rwlock_collation_exception_list,
 
 my_core::PSI_rwlock_info all_rocksdb_rwlocks[] = {
     {&key_rwlock_collation_exception_list, "collation_exception_list",
-     PSI_FLAG_GLOBAL},
-    {&key_rwlock_read_free_rpl_tables, "read_free_rpl_tables", PSI_FLAG_GLOBAL},
+     PSI_FLAG_SINGLETON},
+    {&key_rwlock_read_free_rpl_tables, "read_free_rpl_tables",
+     PSI_FLAG_SINGLETON},
     {&key_rwlock_skip_unique_check_tables, "skip_unique_check_tables",
-     PSI_FLAG_GLOBAL},
+     PSI_FLAG_SINGLETON},
 };
 
 my_core::PSI_cond_key rdb_signal_bg_psi_cond_key,
     rdb_signal_drop_idx_psi_cond_key, rdb_signal_mc_psi_cond_key;
 
 my_core::PSI_cond_info all_rocksdb_conds[] = {
-    {&rdb_signal_bg_psi_cond_key, "cond signal background", PSI_FLAG_GLOBAL},
+    {&rdb_signal_bg_psi_cond_key, "cond signal background", PSI_FLAG_SINGLETON},
     {&rdb_signal_drop_idx_psi_cond_key, "cond signal drop index",
-     PSI_FLAG_GLOBAL},
+     PSI_FLAG_SINGLETON},
     {&rdb_signal_mc_psi_cond_key, "cond signal manual compaction",
-     PSI_FLAG_GLOBAL},
+     PSI_FLAG_SINGLETON},
 };
 
 my_core::PSI_memory_key rdb_datadic_memory_key, rdb_open_tables_memory_key,
@@ -120,7 +124,7 @@ void init_rocksdb_psi_keys() {
   count = array_elements(all_rocksdb_memory);
   mysql_memory_register(category, all_rocksdb_memory, count);
 }
-#else  // HAVE_PSI_INTERFACE
+#else   // HAVE_PSI_INTERFACE
 void init_rocksdb_psi_keys() {}
 #endif  // HAVE+PSI_INTERFACE
 
